/*
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************

 Application Name     -   MQTT Client
 Application Overview -   The device is running a MQTT client which is
                           connected to the online broker. Three LEDs on the 
                           device can be controlled from a web client by
                           publishing msg on appropriate topics. Similarly, 
                           message can be published on pre-configured topics
                           by pressing the switch buttons on the device.
 
 Application Details  - Refer to 'MQTT Client' README.html
 
 *****************************************************************************/
//*****************************************************************************
//
//! \addtogroup mqtt_server
//! @{
//
//*****************************************************************************
/* Standard includes                                                          */
#include <stdlib.h>
#include <time.h>

/* Hardware includes                                                          */
#include <ti/devices/cc32xx/inc/hw_types.h>
#include <ti/devices/cc32xx/inc/hw_ints.h>
#include <ti/devices/cc32xx/inc/hw_memmap.h>

/* Driverlib includes                                                         */
#include <ti/devices/cc32xx/driverlib/rom.h>
#include <ti/devices/cc32xx/driverlib/rom_map.h>
#include <ti/devices/cc32xx/driverlib/timer.h>

/* TI-Driver includes                                                         */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>

/* Simplelink includes                                                        */
#include <ti/drivers/net/wifi/simplelink.h>

/* MQTT Library includes                                                      */
#include <ti/net/mqtt/mqtt_client.h>

/* Common interface includes                                                  */
#include "network_if.h"
#include "uart_term.h"

#include <math.h>

/* Application includes                                                       */
#include "Board.h"
#include "client_cbs.h"
#include "pthread.h"
#include "mqueue.h"
#include "time.h"
#include "unistd.h"
#include "payload.h"
#include "PLL.h"
#include "adc_single_channel.h"

//*****************************************************************************
//                          LOCAL DEFINES
//*****************************************************************************
/* enables secured client                                                     */
//#define SECURE_CLIENT

/* enables client authentication by the server                                */
//#define CLNT_USR_PWD

#define CLIENT_INIT_STATE        (0x01)
#define MQTT_INIT_STATE          (0x04)

#define APPLICATION_VERSION      "1.0.0"
#define APPLICATION_NAME         "RadioHound Demo"

/* Operate Lib in MQTT 3.1 mode.                                              */
#define MQTT_3_1_1               false
#define MQTT_3_1                 true

#define WILL_TOPIC               "Client"
#define WILL_MSG                 "Client Stopped"
#define WILL_QOS                 MQTT_QOS_2
#define WILL_RETAIN              false

/* Defining Broker IP address and port Number                                 */
//#define SERVER_ADDRESS           "messagesight.demos.ibm.com"
#define SERVER_ADDRESS           "fmnc.cse.nd.edu"
#define SERVER_IP_ADDRESS        "129.74.152.96"
#define PORT_NUMBER              1883
#define SECURED_PORT_NUMBER      8883
#define LOOPBACK_PORT            1882

/* Clean session flag                                                         */
#define CLEAN_SESSION            true

/* Retain Flag. Used in publish message.                                      */
#define RETAIN_ENABLE            1

/* Defining Number of subscription topics                                     */
#define SUBSCRIPTION_TOPIC_COUNT 4

/* Defining Subscription Topic Values                                         */
#define SUBSCRIPTION_TOPIC0      "radiohound/clients/command_to_all/"
#define SUBSCRIPTION_TOPIC1      "radiohound/clients/command/9884e3f6643f"
#define SUBSCRIPTION_TOPIC2      "radiohound/management"    // add a backslash (/) after to enable
#define SUBSCRIPTION_TOPIC3      "radiohound/placeholder"

/* Defining Publish Topic Values                                              */
#define PUBLISH_TOPIC0           "radiohound/clients/announce/9884e3f6643f"
#define PUBLISH_TOPIC0_DATA      "{\"message\": \"ANNOUNCE\", \"payload\": {\"manet\": false, \"IP_addr\": \"10.13.75.97\", \"short_name\": \"HoundDoggies\", \"rh_hardware_version\": \"senior_design\", \"system_version\": \"TI-RTOS\", \"altitude\": 42.0, \"hostname\": \"LaunchPad\", \"master_controller\": \"fmnc.cse.nd.edu\", \"hardware_version\": \"a02082\", \"longitude\": -86.099036999999996, \"group_name\": \"Senior Design\", \"rh_hardware_attached\": 1, \"disk_free\": \"403M\", \"disk_used\": \"5.3G\", \"mac_address\": \"9884e3f6643f\", \"latitude\": 41.539491000000001, \"rh_software_version\": \"custom\", \"check_in_server\": \"None\"}}"

#define PUBLISH_TOPIC1           "radiohound/clients/data/9884e3f6643f"

/* Spawn task priority and Task and Thread Stack Size                         */
#define TASKSTACKSIZE            2048
#define RXTASKSIZE               4096
#define MQTTTHREADSIZE           2048
#define SPAWN_TASK_PRIORITY      9

/* secured client requires time configuration, in order to verify server      */
/* certificate validity (date).                                               */

/* Day of month (DD format) range 1-31                                        */
#define DAY                      1
/* Month (MM format) in the range of 1-12                                     */
#define MONTH                    5
/* Year (YYYY format)                                                         */
#define YEAR                     2017
/* Hours in the range of 0-23                                                 */
#define HOUR                     12
/* Minutes in the range of 0-59                                               */
#define MINUTES                  33
/* Seconds in the range of 0-59                                               */
#define SEC                      21   

/* Number of files used for secure connection                                 */
#define CLIENT_NUM_SECURE_FILES  1

/* Expiration value for the timer that is being used to toggle the Led.       */
#define LED_TIMER_EXPIRATION_VALUE   100 * 1000000

/*  expiration value for the tuning timer   */
#define TUNE_TIMER_EXPIRATION_VALUE   15 * 1000000

/* Expiration value for the timer that is being used for ANNOUNCE packets     */
#define ANN_TIMER_EXPIRATION_VALUE   150 * 1000000

/*  step size of scan in MHz    */
#define SCAN_STEP_SIZE 2

/*  useful for defining buffer lengths  */
#define LEN 150

struct scan_vals
{
    int     fmin;
    int     fmax;
    int     n_samples;
};


//*****************************************************************************
//                      LOCAL FUNCTION PROTOTYPES
//*****************************************************************************
void pushButtonInterruptHandler2(uint_least8_t index);
void pushButtonInterruptHandler3(uint_least8_t index);
void LedTimerPeriodicIntHandler(sigval val);
void AnnTimerPeriodicIntHandler(sigval val);
void LedTimerConfigNStart();
void LedTimerDeinitStop();
static void DisplayBanner(char * AppName);
void *MqttClient(void *pvParameters);
void Mqtt_ClientStop(uint8_t disconnect);
void Mqtt_ServerStop();
void Mqtt_Stop();
void Mqtt_start();
void ParseCommand(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
struct scan_vals initScan(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
int32_t Mqtt_IF_Connect();
int32_t MqttServer_start();
int32_t MqttClient_start();
int32_t MQTT_SendMsgToQueue(struct msgQueue *queueElement);
void ParseTime(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
void CommandToAll(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);

uint8_t * Scan(int fmin_val, int fmax_val, int n_samples);
char * returnScanData(uint8_t * data, int fmin, int fmax, int n_samples);
void setTime();
void SendScanData(char * payload);
void ScanHelper(int fmin_val, int fmax_val, int n_samples_val);



//*****************************************************************************
//                 GLOBAL VARIABLES
//*****************************************************************************

/* Connection state: (0) - connected, (negative) - disconnected               */
int32_t gApConnectionState = -1;
uint32_t gInitState = 0;
uint32_t memPtrCounterfree = 0;
bool     gResetApplication = false;
static NetAppHandle_t gMqttClient;
MQTTClient_Attrib_t  MqttClientExmple_params;
unsigned short g_usTimerInts;
unsigned short ann_usTimerInts;
unsigned short tune_usTimerInts;
unsigned short tune_flag = 0;

/* Receive task handle                                                        */
pthread_t g_rx_task_hndl = (pthread_t) NULL;
uint32_t gUiConnFlag = 0;

/* AP Security Parameters                                                     */
SlWlanSecParams_t SecurityParams = { 0 };

/* Client ID                                                                  */
/* If ClientId isn't set, the MAC address of the device will be copied into   */
/* the ClientID parameter.                                                    */
char ClientId[13] = {'\0'};

/* Client User Name and Password                                              */
const char *ClientUsername = "username1";
const char *ClientPassword = "pwd1";

/* Subscription topics and qos values                                         */
char *topic[SUBSCRIPTION_TOPIC_COUNT] = 
        { SUBSCRIPTION_TOPIC0, SUBSCRIPTION_TOPIC1, \
          SUBSCRIPTION_TOPIC2, SUBSCRIPTION_TOPIC3 };

unsigned char qos[SUBSCRIPTION_TOPIC_COUNT] =
        { MQTT_QOS_2, MQTT_QOS_2, MQTT_QOS_2, MQTT_QOS_2 };

/* Publishing topics and messages                                             */
const char *publish_topic = { PUBLISH_TOPIC0 };
const char *publish_data = { PUBLISH_TOPIC0_DATA };

const char *publish_payload = { PUBLISH_TOPIC1 };
const char *publish_payload_data = {0};

/* Message Queue                                                              */
mqd_t g_PBQueue;
pthread_t mqttThread = (pthread_t) NULL;
pthread_t appThread = (pthread_t) NULL;
timer_t g_timer;
timer_t ann_timer;
uint8_t ann_timer_helper = 0;
timer_t tune_timer;


/* Printing new line                                                          */
char lineBreak[] = "\n\r";

/*  Scan data   */
void * scan_data = NULL;
SPI_Handle PLL_SPI = NULL;

/*  timestamp variables */
SlDateTime_t dateTime = {0};



//*****************************************************************************
//                 Banner VARIABLES
//*****************************************************************************
#ifdef  SECURE_CLIENT

char *Mqtt_Client_secure_files[CLIENT_NUM_SECURE_FILES] = {"ca-cert.pem"};

/* Initialization structure to be used with sl_ExtMqtt_Init API. In order to  */ 
/* use secured socket method, the flag MQTTCLIENT_NETCONN_SEC, cipher,        */
/* n_files and secure_files must be configured.                               */
/* certificates also must be programmed  ("ca-cert.pem").                     */
/* The first parameter is a bit mask which configures server address type and */
/* security mode.                                                             */
/* Server address type: IPv4, IPv6 and URL must be declared with The          */
/* corresponding flag.                                                        */
/* Security mode: The flag MQTTCLIENT_NETCONN_SEC enables the security (TLS)  */
/* which includes domain name verification and certificate catalog            */
/* verification, those verifications can be disabled by adding to the bit mask*/
/* MQTTCLIENT_NETCONN_SKIP_DOMAIN_NAME_VERIFICATION and                       */
/* MQTTCLIENT_NETCONN_SKIP_CERTIFICATE_CATALOG_VERIFICATION flags             */
/* Example: MQTTCLIENT_NETCONN_IP6 | MQTTCLIENT_NETCONN_SEC |                 */
/* MQTTCLIENT_NETCONN_SKIP_CERTIFICATE_CATALOG_VERIFICATION                   */
/* For this bit mask, the IPv6 address type will be in use, the security      */
/* feature will be enable and the certificate catalog verification will be    */
/* skipped.                                                                   */
/* Note: The domain name verification requires URL Server address type        */
/*       otherwise, this verification will be disabled.                       */
MQTTClient_NetAppConnParams_t Mqtt_ClientCtx =
{
     MQTTCLIENT_NETCONN_IP4 | MQTTCLIENT_NETCONN_SEC,
     SERVER_IP_ADDRESS, //SERVER_ADDRESS,
     SECURED_PORT_NUMBER,//  PORT_NUMBER
     SL_SO_SEC_METHOD_SSLv3_TLSV1_2,
     SL_SEC_MASK_SECURE_DEFAULT,
     CLIENT_NUM_SECURE_FILES,
     Mqtt_Client_secure_files
};


#else
MQTTClient_NetAppConnParams_t Mqtt_ClientCtx =
{
    MQTTCLIENT_NETCONN_URL,
    SERVER_ADDRESS,
    PORT_NUMBER, 0, 0, 0,
    NULL
};
#endif

/* Initialize the will_param structure to the default will parameters         */
MQTTClient_Will_t will_param =
{
    WILL_TOPIC,
    WILL_MSG,
    WILL_QOS,
    WILL_RETAIN 
};

//*****************************************************************************
//
//! MQTT_SendMsgToQueue - Utility function that receive msgQueue parameter and 
//! tries to push it the queue with minimal time for timeout of 0.
//! If the queue isn't full the parameter will be stored and the function
//! will return 0.
//! If the queue is full and the timeout expired (because the timeout parameter
//! is 0 it will expire immediately), the parameter is thrown away and the
//! function will return -1 as an error for full queue.
//!
//! \param[in] struct msgQueue *queueElement
//!
//! \return 0 on success, -1 on error
//
//*****************************************************************************
int32_t MQTT_SendMsgToQueue(struct msgQueue *queueElement)
{
    struct timespec abstime = {0};

    clock_gettime(CLOCK_REALTIME, &abstime);

    if(g_PBQueue)
    {
        /* send message to the queue                                         */
        if (mq_timedsend(g_PBQueue, (char *) queueElement, sizeof(struct msgQueue), 0, &abstime) == 0)
        {
            return 0;
        }
    }
    return -1;
}

//*****************************************************************************
//
//! Push Button Handler1(GPIOSW2). Press push button1 (GPIOSW2) Whenever user
//! wants to publish a message. Write message into message queue signaling the
//! event publish messages
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void pushButtonInterruptHandler2(uint_least8_t index)
{
    struct msgQueue queueElement;

    /* Disable the SW2 interrupt */
    GPIO_disableInt(Board_BUTTON0); // SW2

    queueElement.event = PUBLISH_PUSH_BUTTON_PRESSED;
    queueElement.msgPtr = NULL;

    /* write message indicating publish message                              */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full\n\n\r");
    }

}

//*****************************************************************************
//
//! Push Button Handler2(GPIOSW3). Press push button3 Whenever user wants to
//! disconnect from the remote broker. Write message into message queue
//! indicating disconnect from broker.
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void pushButtonInterruptHandler3(uint_least8_t index)
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    queueElement.event = DISC_PUSH_BUTTON_PRESSED;
    queueElement.msgPtr = NULL;

    /* write message indicating disconnect push button pressed message       */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }
}

//*****************************************************************************
//
//! Periodic LED Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void LedTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    g_usTimerInts++;

    if ((g_usTimerInts & 0x1))
    {
        /* Turn Led Off                                                       */
        GPIO_write(Board_LED0, Board_LED_OFF);
    }
    else
    {
        /* Turn Led On                                                        */
        GPIO_write(Board_LED0, Board_LED_ON);
    }
}

//*****************************************************************************
//
//! Periodic Announce Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void AnnTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    ann_usTimerInts++;

    /* check if we've overflowed back to zero, iteratin=g through this counter
     * a set number of times (255?)
     */

    if ((ann_usTimerInts & 0x40)) {
        /* Toggle LED        */
        GPIO_toggle(Board_LED1);

        /* Send Announce packet to RadioHound                                     */
        struct msgQueue queueElement;

        queueElement.event = PUBLISH_PUSH_BUTTON_PRESSED;
        queueElement.msgPtr = NULL;

        /* write message indicating publish message                              */
        if (MQTT_SendMsgToQueue(&queueElement))
        {
            UART_PRINT("\n\n\rQueue is full\n\n\r");
        }

        ann_usTimerInts = 0;
    }


}

/*
 Returns the current time.  Not currently useful..
*/
char *time_stamp() {
    char *timestamp = (char *)malloc(sizeof(char) * 25);

    sprintf(timestamp,"%04d-%02d-%02dT%02d-%02d-%02d", dateTime.tm_year,dateTime.tm_mon,dateTime.tm_day,
            dateTime.tm_hour,dateTime.tm_min,dateTime.tm_sec);
    return timestamp;
}


//*****************************************************************************
//
//! Tune Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void TuneTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    tune_usTimerInts++;

    /* check if we've overflowed back to zero, iteratin=g through this counter
     * a set number of times (255?)
     */

    if ((tune_usTimerInts++ & 0x1)) {

        /*  set global variable to continue tuning PLL  */
        tune_flag = 1;

        GPIO_toggle(Board_LED0);


        tune_usTimerInts = 0;
    }


}

void getSystemTime() {
    SlDateTime_t dateTime =  {0};
     _i16 configLen = sizeof(SlDateTime_t);
     _i8 configOpt = SL_DEVICE_GENERAL_DATE_TIME;
     sl_DeviceGet(SL_DEVICE_GENERAL,&configOpt, &configLen,(_u8 *)(&dateTime));

     UART_PRINT("Day %d,Mon %d,Year %d,Hour %d,Min %d,Sec %d\n\n\r",dateTime.tm_day,dateTime.tm_mon,dateTime.tm_year,
             dateTime.tm_hour,dateTime.tm_min,dateTime.tm_sec);
}

void setTime()
{
    sl_DeviceSet(SL_DEVICE_GENERAL, SL_DEVICE_GENERAL_DATE_TIME, sizeof(SlDateTime_t), (uint8_t *)(&dateTime));
}


//*****************************************************************************
//
//! Function to configure and start timer to blink the LED while device is
//! trying to connect to an AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void LedTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &LedTimerPeriodicIntHandler;
    timer_create(2, &sev, &g_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = LED_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = LED_TIMER_EXPIRATION_VALUE;

    timer_settime(g_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Disable the LED blinking Timer as Device is connected to AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void LedTimerDeinitStop()
{ 
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(g_timer);
}

//*****************************************************************************
//
//! Function to configure and start timer to blink the LED while device is
//! trying to connect to an AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void TuneTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &TuneTimerPeriodicIntHandler;
    timer_create(2, &sev, &tune_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = TUNE_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = TUNE_TIMER_EXPIRATION_VALUE;

    timer_settime(tune_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Disable the LED blinking Timer as Device is connected to AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void TuneTimerDeinitStop()
{
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(tune_timer);
}


//*****************************************************************************
//
//! Function to configure and start timer to send ANNOUNCE packet every 10 sec
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void AnnounceTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &AnnTimerPeriodicIntHandler;
    timer_create(2, &sev, &ann_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = ANN_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = ANN_TIMER_EXPIRATION_VALUE;

    timer_settime(ann_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Turn off ANNOUNCE timer for communicating node status
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void AnnounceTimerDeinitStop()
{
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(ann_timer);
}

uint32_t ReadADC() {
    uint16_t     i;
    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;
    uint32_t avgVal;
    uint32_t chunk = ADC_SAMPLE_COUNT/TotalAverages;
    uint32_t totalVolts = 0;

    ADC_Params_init(&params);
    adc = ADC_open(Board_ADC1, &params);

    if (adc == NULL) {
        UART_PRINT("Error initializing ADC channel 1\n");
        while (1);
    }
    /* Modified Code for averaging */
    for (i = 1; i < ADC_SAMPLE_COUNT+1; i++) {
        res = ADC_convert(adc, &adcValue1[i]);

        if (res == ADC_STATUS_SUCCESS) {

            adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);
            totalVolts = totalVolts + adcValue1MicroVolt[i];

            if (i % chunk == 0){
                avgVal = totalVolts/chunk;
                UART_PRINT("ADC channel 1 convert result (%d): %d uV\n\r", i, avgVal);
                totalVolts = 0;
            }
        }
        else {
            UART_PRINT("ADC channel 1 convert failed (%d)\n", i);
        }
        /* Original Example Code
         * for (i = 0; i < ADC_SAMPLE_COUNT; i++) {
            res = ADC_convert(adc, &adcValue1[i]);

            if (res == ADC_STATUS_SUCCESS) {

                adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);

                Display_printf(display, 0, 0, "ADC channel 1 convert result (%d): %d uV\n", i,
                                   adcValue1MicroVolt[i]);
            }
            else {
                Display_printf(display, 0, 0, "ADC channel 1 convert failed (%d)\n", i);
         }*/
    }

    ADC_close(adc);

    return (avgVal);
}

void CommandToAll(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * time = "set_time";

    UART_PRINT("\n\nCommand to all received!\n\n\r");

    if (queueElem->msgPtr) {
        if (strstr((const char *)queueElem->msgPtr + payloadOffset, time)) {
            UART_PRINT("Set time request received!\n\n\r");
            ParseTime(queueElem, topicOffset, payloadOffset);
            char * time = time_stamp();
            UART_PRINT("\n\nSystem Time updated to %s\n\r", time);
            free(time);
            getSystemTime();
        }
    }
}


void ParseCommand(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset)
{
    // expecting either send_config or scan.. call either accordingly

    // send_config (function should also be called by timer periodically
    // (build config update, then send over MQTT queue

    // scan(start_freq, stop_freq)
    char * scan = "scan";
    char * config = "send_config";

    UART_PRINT("\n\n\rParsing Message..\n\r");
    //UART_PRINT("Payload being parsed: %s\n\n\r", (char *)queueElem->msgPtr + payloadOffset);

    if (queueElem->msgPtr) {
        if (strstr((const char *)queueElem->msgPtr + payloadOffset, scan)) {
            UART_PRINT("Scan job received!\n\r");
            struct scan_vals scan_job = initScan(queueElem, topicOffset, payloadOffset);
            ScanHelper(scan_job.fmin, scan_job.fmax, scan_job.n_samples);
        } else if (strstr((const char*)queueElem->msgPtr + payloadOffset, config)) {
            UART_PRINT("Config requested!");
        } else {
            UART_PRINT("Unknown request from backend.");
        }
    } else {
        UART_PRINT("pointer to NULL :(");
    }


    return;
}


void ParseTime(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * hour_start;
    char * hour_stop;

    char * tmp;
    int offset = 0;

    char * time_start;

    char * day_start;
    char * day_stop;

    char * year_start;
    char * year_stop;

    char * timezone_start;
    char * timezone_stop;

    char * minute_start;
    char * minute_stop;

    char * second_start;
    char * second_stop;

    char * month_start;
    char * month_stop;

    char * day_month_start;
    char * day_month_stop;

    char * time = "\"time\"";


    /* find the time pointer    */
    time_start = strstr((const char *)queueElem->msgPtr + payloadOffset, time) + 9;
    if (time_start) {

        /*  get the day of the week */
        day_start = time_start;
        tmp = time_start;
        day_stop = strchr((const char*)tmp, ' ');
        *day_stop = 0;
        UART_PRINT("Found day of the week: %s\n\r", day_start);

        offset = day_stop - day_start + 1;

        /*  get the month */
        month_start = time_start + offset;
        tmp = time_start + offset;
        month_stop = strchr((const char*)tmp, ' ');
        *month_stop = 0;
        UART_PRINT("Found month: %s\n\r", month_start);

        offset = month_stop - day_start + 1;

        /*  get the day of the month */
        day_month_start = time_start + offset;
        tmp = time_start + offset;
        day_month_stop = strchr((const char*)tmp, ' ');
        *day_month_stop = 0;
        UART_PRINT("Found day of the month: %s\n\r", day_month_start);

        offset = day_month_stop - day_start + 1;

        /*  get the hour */
        hour_start = time_start + offset;
        tmp = time_start + offset;
        hour_stop = strchr((const char*)tmp, ':');
        *hour_stop = 0;
        UART_PRINT("Found hour: %s\n\r", hour_start);

        offset = hour_stop - day_start + 1;

        /*  get the minute */
        minute_start = time_start + offset;
        tmp = time_start + offset;
        minute_stop = strchr((const char*)tmp, ':');
        *minute_stop = 0;
        UART_PRINT("Found minute: %s\n\r", minute_start);

        offset = minute_stop - day_start + 1;

        /*  get the second */
        second_start = time_start + offset;
        tmp = time_start + offset;
        second_stop = strchr((const char*)tmp, ' ');
        *second_stop = 0;
        UART_PRINT("Found second: %s\n\r", second_start);

        offset = second_stop - day_start + 1;

        /*  get the timezone */
        timezone_start = time_start + offset;
        tmp = time_start + offset;
        timezone_stop = strchr((const char*)tmp, ' ');
        *timezone_stop = 0;
        UART_PRINT("Found time zone: %s\n\r", timezone_start);

        offset = timezone_stop - day_start + 1;

        /*  get the year */
        year_start = time_start + offset;
        tmp = time_start + offset;
        year_stop = strchr((const char*)tmp, '"');
        *year_stop = 0;
        UART_PRINT("Found year: %s\n\r", year_start);

        offset = year_stop - day_start + 1;

        dateTime.tm_day =   (_u32)atoi(day_month_start);          // Day of month (DD format) range 1-31
        dateTime.tm_year =  (_u32)atoi(year_start);        // Year (YYYY format)
        dateTime.tm_hour =  (_u32)atoi(hour_start);          // Hours in the range of 0-23
        dateTime.tm_min =   (_u32)atoi(minute_start);          // Minutes in the range of 0-59
        dateTime.tm_sec =   (_u32)atoi(second_start);          // Seconds in the range of  0-59


        if (!(strcmp(month_start, "January")))
            dateTime.tm_mon = 1;
        else if (!(strcmp(month_start, "February")))
            dateTime.tm_mon = 2;
        else if (!(strcmp(month_start, "March")))
            dateTime.tm_mon = 3;
        else if (!(strcmp(month_start, "April")))
            dateTime.tm_mon = 4;
        else if (!(strcmp(month_start, "May")))
            dateTime.tm_mon = 5;
        else if (!(strcmp(month_start, "June")))
            dateTime.tm_mon = 6;
        else if (!(strcmp(month_start, "July")))
            dateTime.tm_mon = 7;
        else if (!(strcmp(month_start, "August")))
            dateTime.tm_mon = 8;
        else if (!(strcmp(month_start, "September")))
            dateTime.tm_mon = 9;
        else if (!(strcmp(month_start, "October")))
            dateTime.tm_mon = 10;
        else if (!(strcmp(month_start, "November")))
            dateTime.tm_mon = 11;
        else if (!(strcmp(month_start, "December")))
            dateTime.tm_mon = 12;
        else {
            UART_PRINT("Invalid month given: %s\n\n\r", month_start);
            return;
        }

        //  set the system time
        setTime();


    } else {
        UART_PRINT("time not given! Exiting...\n\n\r");
        return;
    }

}



struct scan_vals initScan(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * fmin_start;
    char * fmax_start;

    char * fmin_stop;
    char * fmax_stop;

    char * fmin = "fmin";
    char * fmax = "fmax";
    char * n_samples = "nsamples";

    char * n_samples_start;
    char * n_samples_stop;

    int fmin_val = 0;
    int fmax_val = 0;
    int n_samples_val = 0;

    struct scan_vals return_vals;

    /*  check if the scan request has an n_samples value.. if so, extract and continue    */
    n_samples_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), n_samples) + 10);
    if (n_samples_start) {
        n_samples_stop = strchr((const char *)n_samples_start, ',');
        *n_samples_stop = 0;
        n_samples_val = atoi(n_samples_start);
        UART_PRINT("Found n_samples val: %s\t%d\n\r", n_samples_start, n_samples_val);
    } else {
        UART_PRINT("Scan received without n_samples value, canceling...\n\n\r");
        free(n_samples_start);
        return;
    }

    /* find the fmin pointer    */
    fmin_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), fmin) + 6);
    if (fmin_start) {
        fmin_stop = strchr((const char*)fmin_start, ',');
        *fmin_stop = 0;
        fmin_val = atoi(fmin_start);
        UART_PRINT("Found fmin val: %s\t%d\n\r", fmin_start, fmin_val);
    } else {
        UART_PRINT("FMIN not given! Exiting...\n\n\r");
        free(fmin_start);
        free(n_samples_start);
        return;
    }

    /* find the fmax pointer    */
    fmax_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), fmax) + 6);
    if (fmax_start) {
        fmax_stop = strchr((const char*)fmax_start, ',');
        *fmax_stop = 0;
        fmax_val = atoi(fmax_start);
        UART_PRINT("Found fmax val: %s\t%d\n\r", fmax_start, fmax_val);
    } else {
        UART_PRINT("FMAX not given! Exiting...\n\n\r");
        free(fmin_start);
        free(n_samples_start);
        free(fmax_start);
        return;
    }

    /*  convert ASCII scan values into ints */
    if (!(fmin_val)) {
        /*  given a zero value for frequency    */
        UART_PRINT("Zero freq value received!\n\n\r");
        return;
    }

     if (!(fmax_val)) {
         /*  given a zero value for frequency    */
         UART_PRINT("Zero freq value received!\n\n\r");
         return;
     }

     if (!(n_samples_val)) {
         /*  given a zero value for n_samples    */
         UART_PRINT("Zero freq value received!\n\n\r");
         return;
     }

    UART_PRINT("Scanning from:\n\rMin:\t%d MHz\n\rMax:\t%d MHz\n\rN_Samples:\t%d\n\r",
               fmin_val / 1000000, fmax_val / 1000000, n_samples_val);

    free(fmax_start);
    free(fmin_start);
    free(n_samples_start);

    return_vals.fmax = fmax_val;
    return_vals.fmin = fmin_val;
    return_vals.n_samples = n_samples_val;

    return(return_vals);
}

void ScanHelper(int fmin_val, int fmax_val, int n_samples_val) {
    uint8_t * data = Scan(fmin_val, fmax_val, n_samples_val);
    UART_PRINT("data returned\n\r");
    char * payload_string = returnScanData(data, fmin_val, fmax_val, n_samples_val);
    SendScanData(payload_string);
}

uint8_t * Scan(int fmin_val, int fmax_val, int n_samples) {
    //int actual_freq;
    float dispfreq;
    uint32_t adc_sample;

    uint8_t * data = calloc(1026, sizeof(uint16_t));

    /*  start timer */
    TuneTimerConfigNStart();

    /*  calculate step size in MHz  */
    int step_size = (fmax_val - fmin_val + 1) / n_samples;
    UART_PRINT("%d\n\r", step_size);
    UART_PRINT("Num samples: %d\n\r", n_samples);
    UART_PRINT("Step Size: %d\n\r", step_size);

    /*  scan through stuff  */
    int i = 0;
    int g = 0;
    for (i = fmin_val; i < fmax_val + 1; i++) {
        dispfreq = tune_PLL(i / 1000000, PLL_SPI);
        /*
         *
         while (!GPIO_read(Board_GPIO_12)) {
            dispfreq = tune_PLL(i / 1000000, PLL_SPI);
        }
        */
        //while (!tune_flag){};
        UART_PRINT("Target set:\t%d\n\r", i / 1000000);
        adc_sample = ReadADC();
        data[g] = (uint8_t)(adc_sample / 1000);
        g++;
        //tune_flag = 0;
        i = i + step_size -1;              // 2 MHz steps until changed??
    }

    int j;
    for (j = 0; j < n_samples; j++) {
        UART_PRINT("%02.02f\n\r", 20*log2f((float)data[j] / 1000));
    }

    UART_PRINT("Done\n\r");

    void TuneTimerDeinitStop();
    UART_PRINT("timer off\n\r");

    return data;

}


char * returnScanData(uint8_t * data, int fmin, int fmax, int n_samples) {

    UART_PRINT("entered return data\n\r");

    /*  probably add a bunch of other appended payload vars here before sending using MSG_queue structure of button press   */
    UART_PRINT("Sending data..\n\r");

    char * str1 = "{\"software_version\": \"0.0.1.7\", \"timestamp\": \"";
    char * str2 = "\", \"gain\": 1, \"data\": \"";
    char * str3 = "\", \"short_name\": \"HoundDoggies\", \"altitude\": 2.0, \"uncertainty\": 100, \"batch\": 0, \"longitude\":  -86.099036999999996, \"sample_rate\": 2560000.0, \"mac_address\": \"9884e3f6643f\", \"latitude\": 41.539491000000001, \"center_frequency\": ";
    char * str4 = ", \"metadata\": {\"fmin\":";
    char * str5 = ", \"data_type\": \"periodogram\", \"fmax\": ";
    char * str6 = ", \"xcount\": ";
    char * str7 = ", \"xstop\": ";
    char * str8 = ", \"nfft\": ";
    char * str9 = ", \"xstart\": ";

    UART_PRINT("Definitions done\n\r");

    char * payload = malloc(5000*sizeof(char));
    if (payload) {
        UART_PRINT("memory for payload received.\n\r");
    } else {
        UART_PRINT("did not get payload\n\r");
    }
    char * tmp = malloc(500*sizeof(char));
    if (tmp) {
        UART_PRINT("memory for tmp received.\n\r");
    } else {
        UART_PRINT("did not get tmp\n\r");
    }


    sprintf(tmp, "%s%s%s", str1, time_stamp(), str2);
    UART_PRINT("%s\n\r", tmp);
    strcat(payload, tmp);
    UART_PRINT("%s\n\r", payload);

    int i;
    for (i = 0; i < n_samples; i++){
        sprintf(tmp, "%d, ", data[i]);
        UART_PRINT("For %d\t%s\n\r", i, tmp);
        strcat(payload, tmp);
    }

    free(data);

    UART_PRINT("Data converted to string.\n\r");

    sprintf(tmp, "%s%d%s%d%s%d%s%d%s%d%s%d%s%d}}", str3, fmax, str4, fmin, str5, fmax,
            str6, n_samples, str7, fmax, str8, n_samples, str9, fmin);
    strcat(payload, tmp);

    UART_PRINT("Payload:\n\n\r%s\n\n\r", payload);

    return(payload);
}


void SendScanData(char * payload) {
    /* Send Announce packet to RadioHound                                     */
    struct msgQueue queueElement;

    publish_payload_data = payload;

    queueElement.event = PUBLISH_PUSH_BUTTON_PRESSED;
    queueElement.msgPtr = payload;

    /* write message indicating publish message                              */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full\n\n\r");
    }
}





//*****************************************************************************
//
//! Application startup display on UART
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
static void DisplayBanner(char * AppName)
{
    UART_PRINT("\n\n\n\r");
    UART_PRINT("\t\t *************************************************\n\r");
    UART_PRINT("\t\t    CC3220 %s Application       \n\r", AppName);
    UART_PRINT("\t\t *************************************************\n\r");
    UART_PRINT("\n\n\n\r");
}

void *MqttClientThread(void * pvParameters)
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    MQTTClient_run((NetAppHandle_t)pvParameters);

    queueElement.event  = LOCAL_CLIENT_DISCONNECTION;
    queueElement.msgPtr = NULL;

    /* write message indicating disconnect Broker message.                    */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }

    pthread_exit(0);

    return NULL;
}

//*****************************************************************************
//
//! Task implementing MQTT Server plus client bridge
//!
//! This function
//!    1. Initializes network driver and connects to the default AP
//!    2. Initializes the mqtt client ans server libraries and set up MQTT
//!       with the remote broker.
//!    3. set up the button events and their callbacks(for publishing)
//!    4. handles the callback signals
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
void * MqttClient(void *pvParameters)
{

    struct msgQueue queueElemRecv;
    long lRetVal = -1;
    char *tmpBuff;

    /* Initializing Client and Subscribing to the Broker.                     */
    if (gApConnectionState >= 0)
    {
        lRetVal = MqttClient_start();
        if (lRetVal == -1)
        {
            UART_PRINT("MQTT Client lib initialization failed\n\r");
            pthread_exit(0);
            return NULL;
        }
    }

    /* handling the signals from various callbacks including the push button  */
    /* prompting the client to publish a msg on PUB_TOPIC OR msg received by  */
    /* the server on enrolled topic(for which the on-board client ha enrolled)*/
    /* from a local client(will be published to the remote broker by the      */
    /* client) OR msg received by the client from the remote broker (need to  */
    /* be sent to the server to see if any local client has subscribed on the */
    /* same topic).                                                           */
    for (;;)
    {
        /* waiting for signals                                                */
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);

        switch (queueElemRecv.event)
        {
            case PUBLISH_PUSH_BUTTON_PRESSED:

                /* send publish message                                       */
                lRetVal = MQTTClient_publish(gMqttClient, (char*) publish_topic, strlen((char*)publish_topic), (char*)publish_data, strlen((char*) publish_data), MQTT_QOS_2 | ((RETAIN_ENABLE)?MQTT_PUBLISH_RETAIN:0) );

                UART_PRINT("\n\r CC3200 Publishes the following message \n\r");
                UART_PRINT("Topic: %s\n\r", publish_topic);
                UART_PRINT("Data: %s\n\r", publish_data);

                /* Clear and enable again the SW2 interrupt */
                GPIO_clearInt(Board_BUTTON0); // SW2
                GPIO_enableInt(Board_BUTTON0); // SW2

                break;

                /* msg received by client from remote broker (on a topic      */
                /* subscribed by local client)                                */
            case PAYLOAD_SEND:

                /* send publish message                                       */
                lRetVal = MQTTClient_publish(gMqttClient, (char*) publish_payload, strlen((char*)publish_payload), (char*)publish_payload_data, strlen((char*) publish_payload_data), MQTT_QOS_2 | ((RETAIN_ENABLE)?MQTT_PUBLISH_RETAIN:0) );

                UART_PRINT("\n\r CC3200 Publishes the following message \n\r");
                UART_PRINT("Topic: %s\n\r", publish_topic);
                UART_PRINT("Data: %s\n\r", publish_data);

                break;

                /* msg received by client from remote broker (on a topic      */
                /* subscribed by local client)                                */
            case MSG_RECV_BY_CLIENT:
                tmpBuff = (char *) ((char *) queueElemRecv.msgPtr + 12);
                if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC1, queueElemRecv.topLen) == 0)
                {
                    /* for manipulating payload data    */
                    uint32_t topicOffset = 0;
                    uint32_t payloadOffset = 0;

                    /*  toggle LED to indicate received message */
                    GPIO_toggle(Board_LED0);

                    /*  set the topic and payload offsets   */
                    topicOffset = sizeof(struct publishMsgHeader);
                    payloadOffset = sizeof(struct publishMsgHeader) + queueElemRecv.topLen + 1;

                    ParseCommand(&queueElemRecv, topicOffset, payloadOffset);        // either send config or scan
                }
                /*  this is where handling of other events will occur   */
                else if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC0, queueElemRecv.topLen) == 0)
                {
                    /*  this is a command to all, hopefully to set the current time */

                    /* for manipulating payload data    */
                    uint32_t topicOffset = 0;
                    uint32_t payloadOffset = 0;

                    GPIO_toggle(Board_LED1);

                    /*  set the topic and payload offsets   */
                    topicOffset = sizeof(struct publishMsgHeader);
                    payloadOffset = sizeof(struct publishMsgHeader) + queueElemRecv.topLen + 1;

                    CommandToAll(&queueElemRecv, topicOffset, payloadOffset);    // set system time
                }
                /*
                else if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC2, queueElemRecv.topLen) == 0)
                {
                    GPIO_toggle(Board_LED2);
                }
                */

                free(queueElemRecv.msgPtr);
                break;

                /* On-board client disconnected from remote broker, only      */
                /* local MQTT network will work                               */
            case LOCAL_CLIENT_DISCONNECTION:
                UART_PRINT("\n\rOn-board Client Disconnected\n\r\r\n");
                gUiConnFlag = 0;
                break;

                /* Push button for full restart check                         */
            case DISC_PUSH_BUTTON_PRESSED:
                gResetApplication = true;
                break;
                
            case THREAD_TERMINATE_REQ:
                gUiConnFlag = 0;
                pthread_exit(0);
                return NULL;
                
            default:
                sleep(1);
                break;
        }
    }
}

//*****************************************************************************
//
//! This function connect the MQTT device to an AP with the SSID which was 
//! configured in SSID_NAME definition which can be found in Network_if.h file, 
//! if the device can't connect to to this AP a request from the user for other 
//! SSID will appear.
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
int32_t Mqtt_IF_Connect()
{
    int32_t lRetVal;
    char SSID_Remote_Name[32];
    int8_t Str_Length;

    memset(SSID_Remote_Name, '\0', sizeof(SSID_Remote_Name));
    Str_Length = strlen(SSID_NAME);

    if (Str_Length)
    {
        /* Copy the Default SSID to the local variable                        */
        strncpy(SSID_Remote_Name, SSID_NAME, Str_Length);
    }

    /* Display Application Banner                                             */
    DisplayBanner(APPLICATION_NAME);

    GPIO_write(Board_LED0, Board_LED_OFF);
    GPIO_write(Board_LED1, Board_LED_OFF);
    GPIO_write(Board_LED2, Board_LED_OFF);

    /* Reset The state of the machine                                         */
    Network_IF_ResetMCUStateMachine();

    /* Start the driver                                                       */
    lRetVal = Network_IF_InitDriver(ROLE_STA);
    if (lRetVal < 0)
    {
        UART_PRINT("Failed to start SimpleLink Device\n\r", lRetVal);
        return -1;
    }

    /* switch on Green LED to indicate Simplelink is properly up.             */
    GPIO_write(Board_LED2, Board_LED_ON);

    /* Start Timer to blink Red LED till AP connection                        */
    LedTimerConfigNStart();

    /* Initialize AP security params                                          */
    SecurityParams.Key = (signed char *) SECURITY_KEY;
    SecurityParams.KeyLen = strlen(SECURITY_KEY);
    SecurityParams.Type = SECURITY_TYPE;

    /* Connect to the Access Point                                            */
    lRetVal = Network_IF_ConnectAP(SSID_Remote_Name, SecurityParams);
    if (lRetVal < 0)
    {
        UART_PRINT("Connection to an AP failed\n\r");
        return -1;
    }

    /* Disable the LED blinking Timer as Device is connected to AP.           */
    LedTimerDeinitStop();

    /* Switch ON RED LED to indicate that Device acquired an IP.              */
    GPIO_write(Board_LED0, Board_LED_ON);

    sleep(1);

    /* Enable ANNOUNCE timer to send packets to RadioHound backend every 10 sec */
    AnnounceTimerConfigNStart();

    GPIO_write(Board_LED0, Board_LED_OFF);
    GPIO_write(Board_LED1, Board_LED_OFF);
    GPIO_write(Board_LED2, Board_LED_OFF);

    return 0;
}

//*****************************************************************************
//!
//! MQTT Start - Initialize and create all the items required to run the MQTT
//! protocol
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
void Mqtt_start()
{
    int32_t threadArg = 100;
    pthread_attr_t pAttrs;
    struct sched_param priParam;
    int32_t retc = 0;
    mq_attr attr;
    unsigned mode = 0;

    /* sync object for inter thread communication                             */
    attr.mq_maxmsg = 10;
    attr.mq_msgsize = sizeof(struct msgQueue);
    g_PBQueue = mq_open("g_PBQueue", O_CREAT, mode, &attr);

    if (g_PBQueue == NULL)
    {
        UART_PRINT("MQTT Message Queue create fail\n\r");
        gInitState &= ~MQTT_INIT_STATE;
        return;
    }

    /* Set priority and stack size attributes                                 */
    pthread_attr_init(&pAttrs);
    priParam.sched_priority = 2;
    retc = pthread_attr_setschedparam(&pAttrs, &priParam);
    retc |= pthread_attr_setstacksize(&pAttrs, MQTTTHREADSIZE);
    retc |= pthread_attr_setdetachstate(&pAttrs, PTHREAD_CREATE_DETACHED);

    if (retc != 0)
    {
        gInitState &= ~MQTT_INIT_STATE;
        UART_PRINT("MQTT thread create fail\n\r");
        return;
    }

    retc = pthread_create(&mqttThread, &pAttrs, MqttClient, (void *) &threadArg);
    if (retc != 0)
    {
        gInitState &= ~MQTT_INIT_STATE;
        UART_PRINT("MQTT thread create fail\n\r");
        return;
    }

    /* enable interrupt for the GPIO 13 (SW3) and GPIO 22 (SW2).              */
    GPIO_setCallback(Board_BUTTON0, pushButtonInterruptHandler2);
    GPIO_enableInt(Board_BUTTON0); // SW2

    GPIO_setCallback(Board_BUTTON1, pushButtonInterruptHandler3);
    GPIO_enableInt(Board_BUTTON1); // SW3

    gInitState &= ~MQTT_INIT_STATE;

}

//*****************************************************************************
//!
//! MQTT Stop - Close the client instance and free all the items required to 
//! run the MQTT protocol
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************

void Mqtt_Stop()
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    if (gApConnectionState >= 0)
    {
        Mqtt_ClientStop(1);
    }

    queueElement.event = THREAD_TERMINATE_REQ;
    queueElement.msgPtr = NULL;

    /* write message indicating publish message                               */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }

    sleep(2);

    mq_close(g_PBQueue);
    g_PBQueue = NULL;

    sl_Stop(200);
    UART_PRINT("\n\r Client Stop completed\r\n");

    /* Disable the SW2 and SW3 interrupt */
    GPIO_disableInt(Board_BUTTON0); // SW2
    GPIO_disableInt(Board_BUTTON1); // SW3
}

int32_t MqttClient_start()
{
    int32_t lRetVal = -1;
    int32_t iCount = 0;

    int32_t threadArg = 100;
    pthread_attr_t pAttrs;
    struct sched_param priParam;

    MqttClientExmple_params.clientId = ClientId;
    MqttClientExmple_params.connParams = &Mqtt_ClientCtx;
    MqttClientExmple_params.mqttMode31 = MQTT_3_1;
    MqttClientExmple_params.blockingSend = true;

    gInitState |= CLIENT_INIT_STATE;

    /* Initialize MQTT client lib                                             */
    gMqttClient = MQTTClient_create(MqttClientCallback, &MqttClientExmple_params);
    if( gMqttClient == NULL )
    {
        /* lib initialization failed                                          */
        gInitState &= ~CLIENT_INIT_STATE;
        return -1;
    }

    /* Open Client Receive Thread start the receive task. Set priority and    */
    /* stack size attributes                                                  */
    pthread_attr_init(&pAttrs);
    priParam.sched_priority = 2;
    lRetVal = pthread_attr_setschedparam(&pAttrs, &priParam);
    lRetVal |= pthread_attr_setstacksize(&pAttrs, RXTASKSIZE);
    lRetVal |= pthread_attr_setdetachstate(&pAttrs, PTHREAD_CREATE_DETACHED);
    lRetVal |= pthread_create(&g_rx_task_hndl, &pAttrs, MqttClientThread, (void *) &threadArg);
    if (lRetVal != 0)
    {
        UART_PRINT("Client Thread Create Failed failed\n\r");
        gInitState &= ~CLIENT_INIT_STATE;
        return -1;
    }
#ifdef SECURE_CLIENT
    setTime();
#endif

    /* setting will parameters                                                */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_WILL_PARAM, &will_param, sizeof(will_param));

#ifdef CLNT_USR_PWD
    /* Set user name for client connection                                    */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_USER_NAME, (void *)ClientUsername, strlen((char*)ClientUsername));

    /* Set password                                                           */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_PASSWORD, (void *)ClientPassword, strlen((char*)ClientPassword));
#endif
    /* Initiate MQTT Connect                                                  */
    if (gApConnectionState >= 0)
    {
#if CLEAN_SESSION == false
        bool clean = CLEAN_SESSION;
        MQTTClient_set(gMqttClient, MQTT_CLIENT_CLEAN_CONNECT, (void *)&clean, sizeof(bool));
#endif
        /* The return code of MQTTClient_connect is the ConnACK value that
           returns from the server */
        lRetVal = MQTTClient_connect(gMqttClient);

        /* negative lRetVal means error,
           0 means connection successful without session stored by the server,
           greater than 0 means successful connection with session stored by
           the server */
        if (0 > lRetVal)
        {
            /* lib initialization failed                                      */
            UART_PRINT("Connection to broker failed\n\r");

            gUiConnFlag = 0;
        }
        else
        {
            gUiConnFlag = 1;
        }
        /* Subscribe to topics when session is not stored by the server       */
        if ( (gUiConnFlag == 1) && (0 == lRetVal) )
        {
            uint8_t subIndex;
            MQTTClient_SubscribeParams_t subscriptionInfo[SUBSCRIPTION_TOPIC_COUNT];

            for( subIndex = 0; subIndex < SUBSCRIPTION_TOPIC_COUNT; subIndex++ )
            {
                subscriptionInfo[subIndex].topic = topic[subIndex];
                subscriptionInfo[subIndex].qos = qos[subIndex];
            }

            if (MQTTClient_subscribe(gMqttClient , subscriptionInfo, SUBSCRIPTION_TOPIC_COUNT) < 0)
            {
                UART_PRINT("\n\r Subscription Error \n\r");
                MQTTClient_disconnect(gMqttClient);
                gUiConnFlag = 0;
            }
            else
            {
                for (iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++)
                {
                    UART_PRINT("Client subscribed on %s\n\r,", topic[iCount]);
                }
            }
        }
    }

    gInitState &= ~CLIENT_INIT_STATE;

    return 0;
}

//*****************************************************************************
//!
//! MQTT Client stop - Unsubscribe from the subscription topics and exit the 
//! MQTT client lib.
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************

void Mqtt_ClientStop(uint8_t disconnect)
{
    uint32_t iCount;

    MQTTClient_UnsubscribeParams_t subscriptionInfo[SUBSCRIPTION_TOPIC_COUNT];

    for( iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++ )
    {
        subscriptionInfo[iCount].topic = topic[iCount];
    }

    MQTTClient_unsubscribe(gMqttClient , subscriptionInfo, SUBSCRIPTION_TOPIC_COUNT);
    for (iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++)
    {
        UART_PRINT("Unsubscribed from the topic %s\r\n", topic[iCount]);
    }
    gUiConnFlag = 0;

    /* exiting the Client library                                             */
    MQTTClient_delete(gMqttClient);

}

//*****************************************************************************
//!
//! Utility function which prints the borders
//!
//! \param[in] ch  -  hold the charater for the border.
//! \param[in] n   -  hold the size of the border.
//!
//! \return none.
//!
//*****************************************************************************

void printBorder(char ch, int n)
{
    int        i = 0;

    for(i=0; i<n; i++)    putch(ch);
}

//*****************************************************************************
//!
//! Set the ClientId with its own mac address
//! This routine converts the mac address which is given
//! by an integer type variable in hexadecimal base to ASCII
//! representation, and copies it into the ClientId parameter.
//!
//! \param  macAddress  -   Points to string Hex.
//!
//! \return void.
//!
//*****************************************************************************
void SetClientIdNamefromMacAddress(uint8_t *macAddress)
{
    uint8_t Client_Mac_Name[2];
    uint8_t Index;

    /* When ClientID isn't set, use the mac address as ClientID               */
    if (ClientId[0] == '\0')
    {
        /* 6 bytes is the length of the mac address                           */
        for (Index = 0; Index < SL_MAC_ADDR_LEN; Index++)
        {
            /* Each mac address byte contains two hexadecimal characters      */
            /* Copy the 4 MSB - the most significant character                */
            Client_Mac_Name[0] = (macAddress[Index] >> 4) & 0xf;
            /* Copy the 4 LSB - the least significant character               */
            Client_Mac_Name[1] = macAddress[Index] & 0xf;

            if (Client_Mac_Name[0] > 9)
            {
                /* Converts and copies from number that is greater than 9 in  */
                /* hexadecimal representation (a to f) into ascii character   */
                ClientId[2 * Index] = Client_Mac_Name[0] + 'a' - 10;
            }
            else
            {
                /* Converts and copies from number 0 - 9 in hexadecimal       */
                /* representation into ascii character                        */
                ClientId[2 * Index] = Client_Mac_Name[0] + '0';
            }
            if (Client_Mac_Name[1] > 9)
            {
                /* Converts and copies from number that is greater than 9 in  */
                /* hexadecimal representation (a to f) into ascii character   */
                ClientId[2 * Index + 1] = Client_Mac_Name[1] + 'a' - 10;
            }
            else
            {
                /* Converts and copies from number 0 - 9 in hexadecimal       */
                /* representation into ascii character                        */
                ClientId[2 * Index + 1] = Client_Mac_Name[1] + '0';
            }
        }
    }
}

//*****************************************************************************
//!
//! Utility function which Display the app banner
//!
//! \param[in] appName     -  holds the application name.
//! \param[in] appVersion  -  holds the application version.
//!
//! \return none.
//!
//*****************************************************************************

int32_t DisplayAppBanner(char* appName, char* appVersion)
{
    int32_t            ret = 0;
    uint8_t            macAddress[SL_MAC_ADDR_LEN];
    uint16_t           macAddressLen = SL_MAC_ADDR_LEN;
    uint16_t           ConfigSize = 0;
    uint8_t            ConfigOpt = SL_DEVICE_GENERAL_VERSION;
    SlDeviceVersion_t  ver = {0};

    ConfigSize = sizeof(SlDeviceVersion_t);

    /* Print device version info. */
    ret = sl_DeviceGet(SL_DEVICE_GENERAL, &ConfigOpt, &ConfigSize, (uint8_t*)(&ver));

    /* Print device Mac address */
    ret = sl_NetCfgGet(SL_NETCFG_MAC_ADDRESS_GET, 0, &macAddressLen, &macAddress[0]);

    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT("\t   %s Example Ver: %s",appName, appVersion);
    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);
    UART_PRINT("\t CHIP: 0x%x",ver.ChipId);
    UART_PRINT(lineBreak);
    UART_PRINT("\t MAC:  %d.%d.%d.%d",ver.FwVersion[0],ver.FwVersion[1],ver.FwVersion[2],ver.FwVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t PHY:  %d.%d.%d.%d",ver.PhyVersion[0],ver.PhyVersion[1],ver.PhyVersion[2],ver.PhyVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t NWP:  %d.%d.%d.%d",ver.NwpVersion[0],ver.NwpVersion[1],ver.NwpVersion[2],ver.NwpVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t ROM:  %d",ver.RomVersion);
    UART_PRINT(lineBreak);
    UART_PRINT("\t HOST: %s", SL_DRIVER_VERSION);
    UART_PRINT(lineBreak);
    UART_PRINT("\t MAC address: %02x:%02x:%02x:%02x:%02x:%02x", macAddress[0], macAddress[1], macAddress[2], macAddress[3], macAddress[4], macAddress[5]);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);

    SetClientIdNamefromMacAddress(macAddress);

    return ret;
}

void mainThread(void * args)
{

    uint32_t count = 0;
    pthread_t spawn_thread = (pthread_t) NULL;
    pthread_attr_t pAttrs_spawn;
    struct sched_param priParam;
    int32_t retc = 0;
    UART_Handle tUartHndl;

    /* Call driver init functions */
    ADC_init();
    Board_initGPIO();
    Board_initSPI();

    /* Configure the UART                                                     */
    tUartHndl = InitTerm();
    /* remove uart receive from LPDS dependency                               */
    UART_control(tUartHndl, UART_CMD_RXDISABLE, NULL);

    char * timestamp = time_stamp();
    UART_PRINT("Current time: %s\n\n\r", timestamp);
    free(timestamp);


    /*  initialize PLL and relevant things here, found in the main thread of PLL.c  */

    UART_PRINT("Initializing PLL...\n\n\r");

    PLL_SPI = myInitSPI();
    if (PLL_SPI == NULL) {
        UART_PRINT("SPI handle was NULL, exiting... :(");
        return;
    }

    unsigned short checksums[9] = {0};
    float dispfreq;
    // uint32_t adc_sample = 0;
    init_PLL(PLL_SPI, checksums);
    /*  print out the resulting checksums and verify accuracy   */
    char checksum_echo[50] = "Checksums:";
    UART_PRINT("Checksum echo: %s\n\n\r", checksum_echo);
    sprintf(checksum_echo, "%d %d %d %d %d %d %d %d Total: %d\r\n",
            checksums[0], checksums[1], checksums[2], checksums[3],
            checksums[4], checksums[5], checksums[6], checksums[7],
            checksums[8]);
    UART_PRINT("Checksum echo: %s\n\n\r", checksum_echo);

    /*  tune to a starting frequency  (750 - 110 = 640 MHz)  */
    dispfreq = tune_PLL(750, PLL_SPI);
    //UART_PRINT("Current Reading: %d\n\r", GPIO_read(Board_GPIO_12));


    /*
     *
    TuneTimerConfigNStart();
    while (!GPIO_read(Board_GPIO_12)) {
        while(!tune_flag) {}
        dispfreq = tune_PLL(750, PLL_SPI);
        UART_PRINT("Lock not detected\n\r");
        tune_flag = 0;
    }
    */


    /*  print   */
    UART_PRINT("PLL Tuned to %f\n\r", dispfreq);

    /* Create the sl_Task                                                     */
    pthread_attr_init(&pAttrs_spawn);
    priParam.sched_priority = SPAWN_TASK_PRIORITY;
    retc = pthread_attr_setschedparam(&pAttrs_spawn, &priParam);
    retc |= pthread_attr_setstacksize(&pAttrs_spawn, TASKSTACKSIZE);
    retc |= pthread_attr_setdetachstate(&pAttrs_spawn, PTHREAD_CREATE_DETACHED);

    retc = pthread_create(&spawn_thread, &pAttrs_spawn, sl_Task, NULL);

    if (retc != 0)
    {
        UART_PRINT("could not create simplelink task\n\r");
        while (1);
    }

    retc = sl_Start(0, 0, 0);
    if (retc < 0)
    {
        /* Handle Error */
        UART_PRINT("\n sl_Start failed\n");
        while(1);
    }

    /* Output device information to the UART terminal */
    retc = DisplayAppBanner(APPLICATION_NAME, APPLICATION_VERSION);

    retc = sl_Stop(SL_STOP_TIMEOUT );
    if (retc < 0)
    {
        /* Handle Error */
        UART_PRINT("\n sl_Stop failed\n");
        while(1);
    }

    if(retc < 0)
    {
        /* Handle Error */
        UART_PRINT("mqtt_client - Unable to retrieve device information \n");
        while (1);
    }


    while (1)
    {

        gResetApplication = false;
        topic[0] = SUBSCRIPTION_TOPIC0;
        topic[1] = SUBSCRIPTION_TOPIC1;
        topic[2] = SUBSCRIPTION_TOPIC2;
        topic[3] = SUBSCRIPTION_TOPIC3;
        gInitState = 0;

        /* Connect to AP                                                      */
        gApConnectionState = Mqtt_IF_Connect();

        gInitState |= MQTT_INIT_STATE;
        /* Run MQTT Main Thread (it will open the Client and Server)          */
        Mqtt_start();               

        /* Wait for init to be completed!!!                                   */
        while (gInitState != 0)
        {
            UART_PRINT(".");
            sleep(1);
        }
        UART_PRINT(".\r\n");

        while (gResetApplication == false);

        UART_PRINT("TO Complete - Closing all threads and resources\r\n");

        /* Stop the MQTT Process                                              */
        Mqtt_Stop();

        UART_PRINT("reopen MQTT # %d  \r\n", ++count);

    }
}

//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
