/*
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************

 Application Name     -   MQTT Client
 Application Overview -   The device is running a MQTT client which is
                           connected to the online broker. Three LEDs on the 
                           device can be controlled from a web client by
                           publishing msg on appropriate topics. Similarly, 
                           message can be published on pre-configured topics
                           by pressing the switch buttons on the device.
 
 Application Details  - Refer to 'MQTT Client' README.html
 
 *****************************************************************************/
//*****************************************************************************
//
//! \addtogroup mqtt_server
//! @{
//
//*****************************************************************************
/* Standard includes                                                          */
#include <stdlib.h>
#include <time.h>

/* Hardware includes                                                          */
#include <ti/devices/cc32xx/inc/hw_types.h>
#include <ti/devices/cc32xx/inc/hw_ints.h>
#include <ti/devices/cc32xx/inc/hw_memmap.h>

/* Driverlib includes                                                         */
#include <ti/devices/cc32xx/driverlib/rom.h>
#include <ti/devices/cc32xx/driverlib/rom_map.h>
#include <ti/devices/cc32xx/driverlib/timer.h>

/* TI-Driver includes                                                         */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>

/* Simplelink includes                                                        */
#include <ti/drivers/net/wifi/simplelink.h>

/* MQTT Library includes                                                      */
#include <ti/net/mqtt/mqtt_client.h>

/* Common interface includes                                                  */
#include "network_if.h"
#include "uart_term.h"

#include <math.h>

/* Application includes                                                       */
#include "Board.h"
#include "client_cbs.h"
#include "pthread.h"
#include "mqueue.h"
#include "time.h"
#include "unistd.h"
#include "payload.h"
#include "PLL.h"
#include "adc_single_channel.h"

//*****************************************************************************
//                          LOCAL DEFINES
//*****************************************************************************
/* enables secured client                                                     */
//#define SECURE_CLIENT

/* enables client authentication by the server                                */
//#define CLNT_USR_PWD

#define CLIENT_INIT_STATE        (0x01)
#define MQTT_INIT_STATE          (0x04)

#define APPLICATION_VERSION      "1.0.0"
#define APPLICATION_NAME         "RadioHound Demo"

/* Operate Lib in MQTT 3.1 mode.                                              */
#define MQTT_3_1_1               false
#define MQTT_3_1                 true

#define WILL_TOPIC               "Client"
#define WILL_MSG                 "Client Stopped"
#define WILL_QOS                 MQTT_QOS_2
#define WILL_RETAIN              false

/* Defining Broker IP address and port Number                                 */
//#define SERVER_ADDRESS           "messagesight.demos.ibm.com"
#define SERVER_ADDRESS           "fmnc.cse.nd.edu"
#define SERVER_IP_ADDRESS        "129.74.152.96"
#define PORT_NUMBER              1883
#define SECURED_PORT_NUMBER      8883
#define LOOPBACK_PORT            1882

/* Clean session flag                                                         */
#define CLEAN_SESSION            true

/*  demo parameter  */
#define DEMO 0

/* Retain Flag. Used in publish message.                                      */
#define RETAIN_ENABLE            1

/* Defining Number of subscription topics                                     */
#define SUBSCRIPTION_TOPIC_COUNT 4

/* Defining Subscription Topic Values                                         */
#define SUBSCRIPTION_TOPIC0      "radiohound/clients/command_to_all/"
#define SUBSCRIPTION_TOPIC1      "radiohound/clients/command/9884e3f6643f"
#define SUBSCRIPTION_TOPIC2      "radiohound/management"    // add a backslash (/) after to enable
#define SUBSCRIPTION_TOPIC3      "radiohound/placeholder"

/* Defining Publish Topic Values                                              */
#define PUBLISH_TOPIC0           "radiohound/clients/announce/9884e3f6643f"
#define PUBLISH_TOPIC0_DATA      "{\"message\": \"ANNOUNCE\", \"payload\": {\"manet\": false, \"IP_addr\": \"10.13.75.97\", \"short_name\": \"HoundDoggies\", \"rh_hardware_version\": \"senior_design\", \"system_version\": \"TI-RTOS\", \"altitude\": 42.0, \"hostname\": \"LaunchPad\", \"master_controller\": \"fmnc.cse.nd.edu\", \"hardware_version\": \"a02082\", \"longitude\": -86.099036999999996, \"group_name\": \"Senior Design\", \"rh_hardware_attached\": 1, \"disk_free\": \"403M\", \"disk_used\": \"5.3G\", \"mac_address\": \"9884e3f6643f\", \"latitude\": 41.539491000000001, \"rh_software_version\": \"custom\", \"check_in_server\": \"None\"}}"

#define PUBLISH_TOPIC1           "radiohound/clients/data/9884e3f6643f"

/* Spawn task priority and Task and Thread Stack Size                         */
#define TASKSTACKSIZE            2048
#define RXTASKSIZE               2048
#define MQTTTHREADSIZE           2048
#define SPAWN_TASK_PRIORITY      9

/* secured client requires time configuration, in order to verify server      */
/* certificate validity (date).                                               */

/* Day of month (DD format) range 1-31                                        */
#define DAY                      1
/* Month (MM format) in the range of 1-12                                     */
#define MONTH                    5
/* Year (YYYY format)                                                         */
#define YEAR                     2017
/* Hours in the range of 0-23                                                 */
#define HOUR                     12
/* Minutes in the range of 0-59                                               */
#define MINUTES                  33
/* Seconds in the range of 0-59                                               */
#define SEC                      21   

/* Number of files used for secure connection                                 */
#define CLIENT_NUM_SECURE_FILES  1

/* Expiration value for the timer that is being used to toggle the Led.       */
#define LED_TIMER_EXPIRATION_VALUE   100 * 1000000

/*  expiration value for the tuning timer   */
#define TUNE_TIMER_EXPIRATION_VALUE   15 * 1000000

/* Expiration value for the timer that is being used for ANNOUNCE packets     */
#define ANN_TIMER_EXPIRATION_VALUE   150 * 1000000

/*  step size of scan in MHz    */
#define SCAN_STEP_SIZE 2

/*  useful for defining buffer lengths  */
#define LEN 150

struct scan_vals
{
    int     fmin;
    int     fmax;
    int     n_samples;
};


//*****************************************************************************
//                      LOCAL FUNCTION PROTOTYPES
//*****************************************************************************
void pushButtonInterruptHandler2(uint_least8_t index);
void pushButtonInterruptHandler3(uint_least8_t index);
void LedTimerPeriodicIntHandler(sigval val);
void AnnTimerPeriodicIntHandler(sigval val);
void LedTimerConfigNStart();
void LedTimerDeinitStop();
static void DisplayBanner(char * AppName);
void *MqttClient(void *pvParameters);
void Mqtt_ClientStop(uint8_t disconnect);
void Mqtt_ServerStop();
void Mqtt_Stop();
void Mqtt_start();
void ParseCommand(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
struct scan_vals initScan(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
int32_t Mqtt_IF_Connect();
int32_t MqttServer_start();
int32_t MqttClient_start();
int32_t MQTT_SendMsgToQueue(struct msgQueue *queueElement);
void ParseTime(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);
void CommandToAll(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset);

uint8_t * Scan(int fmin_val, int fmax_val, int n_samples);
void buildScanData(uint8_t * data, int fmin, int fmax, int n_samples);
void setTime();
void SendScanData();
void ScanHelper(int fmin_val, int fmax_val, int n_samples_val);



//*****************************************************************************
//                 GLOBAL VARIABLES
//*****************************************************************************

/* Connection state: (0) - connected, (negative) - disconnected               */
int32_t gApConnectionState = -1;
uint32_t gInitState = 0;
uint32_t memPtrCounterfree = 0;
bool     gResetApplication = false;
static NetAppHandle_t gMqttClient;
MQTTClient_Attrib_t  MqttClientExmple_params;
unsigned short g_usTimerInts;
unsigned short ann_usTimerInts;
unsigned short tune_usTimerInts;
unsigned short tune_flag = 0;

/* Receive task handle                                                        */
pthread_t g_rx_task_hndl = (pthread_t) NULL;
uint32_t gUiConnFlag = 0;

/* AP Security Parameters                                                     */
SlWlanSecParams_t SecurityParams = { 0 };

/* Client ID                                                                  */
/* If ClientId isn't set, the MAC address of the device will be copied into   */
/* the ClientID parameter.                                                    */
char ClientId[13] = {'\0'};

/* Client User Name and Password                                              */
const char *ClientUsername = "username1";
const char *ClientPassword = "pwd1";

/* Subscription topics and qos values                                         */
char *topic[SUBSCRIPTION_TOPIC_COUNT] = 
        { SUBSCRIPTION_TOPIC0, SUBSCRIPTION_TOPIC1, \
          SUBSCRIPTION_TOPIC2, SUBSCRIPTION_TOPIC3 };

unsigned char qos[SUBSCRIPTION_TOPIC_COUNT] =
        { MQTT_QOS_2, MQTT_QOS_2, MQTT_QOS_2, MQTT_QOS_2 };

/* Publishing topics and messages                                           DEMO  */
const char *publish_topic = { PUBLISH_TOPIC0 };
const char *publish_data = { PUBLISH_TOPIC0_DATA };

const char *publish_payload = { PUBLISH_TOPIC1 };
char *publish_payload_data = { 0 };

char payload[12000] = {0};
uint8_t data[1026] = {0};
char str_tmp[1500] = {0};

struct msgQueue *globalQueueElem;
uint32_t globalPayloadOffset;
uint32_t globalTopicOffset;
uint8_t scan_flag = 0;



/* Message Queue                                                              */
mqd_t g_PBQueue;
pthread_t mqttThread = (pthread_t) NULL;
pthread_t appThread = (pthread_t) NULL;
timer_t g_timer;
timer_t ann_timer;
uint8_t ann_timer_helper = 0;
timer_t tune_timer;


/* Printing new line                                                          */
char lineBreak[] = "\n\r";

/*  Scan data   */
void * scan_data = NULL;
SPI_Handle PLL_SPI = NULL;

/*  timestamp variables */
SlDateTime_t dateTime = {0};



//*****************************************************************************
//                 Banner VARIABLES
//*****************************************************************************
#ifdef  SECURE_CLIENT

char *Mqtt_Client_secure_files[CLIENT_NUM_SECURE_FILES] = {"ca-cert.pem"};

/* Initialization structure to be used with sl_ExtMqtt_Init API. In order to  */ 
/* use secured socket method, the flag MQTTCLIENT_NETCONN_SEC, cipher,        */
/* n_files and secure_files must be configured.                               */
/* certificates also must be programmed  ("ca-cert.pem").                     */
/* The first parameter is a bit mask which configures server address type and */
/* security mode.                                                             */
/* Server address type: IPv4, IPv6 and URL must be declared with The          */
/* corresponding flag.                                                        */
/* Security mode: The flag MQTTCLIENT_NETCONN_SEC enables the security (TLS)  */
/* which includes domain name verification and certificate catalog            */
/* verification, those verifications can be disabled by adding to the bit mask*/
/* MQTTCLIENT_NETCONN_SKIP_DOMAIN_NAME_VERIFICATION and                       */
/* MQTTCLIENT_NETCONN_SKIP_CERTIFICATE_CATALOG_VERIFICATION flags             */
/* Example: MQTTCLIENT_NETCONN_IP6 | MQTTCLIENT_NETCONN_SEC |                 */
/* MQTTCLIENT_NETCONN_SKIP_CERTIFICATE_CATALOG_VERIFICATION                   */
/* For this bit mask, the IPv6 address type will be in use, the security      */
/* feature will be enable and the certificate catalog verification will be    */
/* skipped.                                                                   */
/* Note: The domain name verification requires URL Server address type        */
/*       otherwise, this verification will be disabled.                       */
MQTTClient_NetAppConnParams_t Mqtt_ClientCtx =
{
     MQTTCLIENT_NETCONN_IP4 | MQTTCLIENT_NETCONN_SEC,
     SERVER_IP_ADDRESS, //SERVER_ADDRESS,
     SECURED_PORT_NUMBER,//  PORT_NUMBER
     SL_SO_SEC_METHOD_SSLv3_TLSV1_2,
     SL_SEC_MASK_SECURE_DEFAULT,
     CLIENT_NUM_SECURE_FILES,
     Mqtt_Client_secure_files
};


#else
MQTTClient_NetAppConnParams_t Mqtt_ClientCtx =
{
    MQTTCLIENT_NETCONN_URL,
    SERVER_ADDRESS,
    PORT_NUMBER, 0, 0, 0,
    NULL
};
#endif

/* Initialize the will_param structure to the default will parameters         */
MQTTClient_Will_t will_param =
{
    WILL_TOPIC,
    WILL_MSG,
    WILL_QOS,
    WILL_RETAIN 
};

//*****************************************************************************
//
//! MQTT_SendMsgToQueue - Utility function that receive msgQueue parameter and 
//! tries to push it the queue with minimal time for timeout of 0.
//! If the queue isn't full the parameter will be stored and the function
//! will return 0.
//! If the queue is full and the timeout expired (because the timeout parameter
//! is 0 it will expire immediately), the parameter is thrown away and the
//! function will return -1 as an error for full queue.
//!
//! \param[in] struct msgQueue *queueElement
//!
//! \return 0 on success, -1 on error
//
//*****************************************************************************
int32_t MQTT_SendMsgToQueue(struct msgQueue *queueElement)
{
    struct timespec abstime = {0};

    clock_gettime(CLOCK_REALTIME, &abstime);

    if(g_PBQueue)
    {
        /* send message to the queue                                         */
        if (mq_timedsend(g_PBQueue, (char *) queueElement, sizeof(struct msgQueue), 0, &abstime) == 0)
        {
            return 0;
        }
    }
    UART_PRINT("Clogged queue! :(\n\r");
    return -1;
}

//*****************************************************************************
//
//! Push Button Handler1(GPIOSW2). Press push button1 (GPIOSW2) Whenever user
//! wants to publish a message. Write message into message queue signaling the
//! event publish messages
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void pushButtonInterruptHandler2(uint_least8_t index)
{
    struct msgQueue queueElement;

    /* Disable the SW2 interrupt */
    GPIO_disableInt(Board_BUTTON0); // SW2

    queueElement.event = PUBLISH_PUSH_BUTTON_PRESSED;
    queueElement.msgPtr = NULL;

    /* write message indicating publish message                              */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full\n\n\r");
    }

}

//*****************************************************************************
//
//! Push Button Handler2(GPIOSW3). Press push button3 Whenever user wants to
//! disconnect from the remote broker. Write message into message queue
//! indicating disconnect from broker.
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void pushButtonInterruptHandler3(uint_least8_t index)
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    queueElement.event = DISC_PUSH_BUTTON_PRESSED;
    queueElement.msgPtr = NULL;

    /* write message indicating disconnect push button pressed message       */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }
}

//*****************************************************************************
//
//! Periodic LED Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void LedTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    g_usTimerInts++;

    if ((g_usTimerInts & 0x1))
    {
        /* Turn Led Off                                                       */
        GPIO_write(Board_LED0, Board_LED_OFF);
    }
    else
    {
        /* Turn Led On                                                        */
        GPIO_write(Board_LED0, Board_LED_ON);
    }
}

//*****************************************************************************
//
//! Periodic Announce Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void AnnTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    ann_usTimerInts++;

    /* check if we've overflowed back to zero, iteratin=g through this counter
     * a set number of times (255?)
     */

    if ((ann_usTimerInts & 0x40)) {
        /* Toggle LED        */
        GPIO_toggle(Board_LED1);

        /* Send Announce packet to RadioHound                                     */
        struct msgQueue queueElement;

        queueElement.event = PUBLISH_PUSH_BUTTON_PRESSED;
        queueElement.msgPtr = NULL;

        /* write message indicating publish message                              */
        if (MQTT_SendMsgToQueue(&queueElement))
        {
            UART_PRINT("\n\n\rQueue is full\n\n\r");
        }

        ann_usTimerInts = 0;
    }


}

/*
 Returns the current time.  Not currently useful..
*/
char *time_stamp() {
    char *timestamp = (char *)malloc(sizeof(char) * 25);

    sprintf(timestamp,"%04d-%02d-%02dT%02d-%02d-%02d", dateTime.tm_year,dateTime.tm_mon,dateTime.tm_day,
            dateTime.tm_hour,dateTime.tm_min,dateTime.tm_sec);
    return timestamp;
}


//*****************************************************************************
//
//! Tune Timer Interrupt Handler
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void TuneTimerPeriodicIntHandler(sigval val)
{
    /* Increment our interrupt counter.                                       */
    tune_usTimerInts++;

    /* check if we've overflowed back to zero, iteratin=g through this counter
     * a set number of times (255?)
     */

    if ((tune_usTimerInts++ & 0x1)) {

        /*  set global variable to continue tuning PLL  */
        tune_flag = 1;

        GPIO_toggle(Board_LED0);


        tune_usTimerInts = 0;
    }


}

void getSystemTime() {
    SlDateTime_t dateTime =  {0};
     _i16 configLen = sizeof(SlDateTime_t);
     _i8 configOpt = SL_DEVICE_GENERAL_DATE_TIME;
     sl_DeviceGet(SL_DEVICE_GENERAL,&configOpt, &configLen,(_u8 *)(&dateTime));

     UART_PRINT("Day %d,Mon %d,Year %d,Hour %d,Min %d,Sec %d\n\n\r",dateTime.tm_day,dateTime.tm_mon,dateTime.tm_year,
             dateTime.tm_hour,dateTime.tm_min,dateTime.tm_sec);
}

void setTime()
{
    sl_DeviceSet(SL_DEVICE_GENERAL, SL_DEVICE_GENERAL_DATE_TIME, sizeof(SlDateTime_t), (uint8_t *)(&dateTime));
}


//*****************************************************************************
//
//! Function to configure and start timer to blink the LED while device is
//! trying to connect to an AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void LedTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &LedTimerPeriodicIntHandler;
    timer_create(2, &sev, &g_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = LED_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = LED_TIMER_EXPIRATION_VALUE;

    timer_settime(g_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Disable the LED blinking Timer as Device is connected to AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void LedTimerDeinitStop()
{ 
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(g_timer);
}

//*****************************************************************************
//
//! Function to configure and start timer to blink the LED while device is
//! trying to connect to an AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void TuneTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &TuneTimerPeriodicIntHandler;
    timer_create(2, &sev, &tune_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = TUNE_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = TUNE_TIMER_EXPIRATION_VALUE;

    timer_settime(tune_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Disable the LED blinking Timer as Device is connected to AP
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void TuneTimerDeinitStop()
{
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(tune_timer);
}


//*****************************************************************************
//
//! Function to configure and start timer to send ANNOUNCE packet every 10 sec
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void AnnounceTimerConfigNStart()
{
    struct itimerspec value;
    sigevent sev;

    /* Create Timer                                                           */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_notify_function = &AnnTimerPeriodicIntHandler;
    timer_create(2, &sev, &ann_timer);

    /* start timer                                                            */
    value.it_interval.tv_sec = 0;
    value.it_interval.tv_nsec = ANN_TIMER_EXPIRATION_VALUE;
    value.it_value.tv_sec = 0;
    value.it_value.tv_nsec = ANN_TIMER_EXPIRATION_VALUE;

    timer_settime(ann_timer, 0, &value, NULL);
}

//*****************************************************************************
//
//! Turn off ANNOUNCE timer for communicating node status
//!
//! \param none
//!
//! return none
//
//*****************************************************************************
void AnnounceTimerDeinitStop()
{
    /* Disable the LED blinking Timer as Device is connected to AP.           */
    timer_delete(ann_timer);
}

uint32_t ReadADC() {
    uint16_t     i;
    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;
    uint32_t avgVal;
    uint32_t chunk = ADC_SAMPLE_COUNT/TotalAverages;
    uint32_t totalVolts = 0;

    ADC_Params_init(&params);
    adc = ADC_open(Board_ADC1, &params);

    if (adc == NULL) {
        UART_PRINT("Error initializing ADC channel 1\n");
        while (1);
    }
    /* Modified Code for averaging */
    for (i = 1; i < ADC_SAMPLE_COUNT+1; i++) {
        res = ADC_convert(adc, &adcValue1[i]);

        if (res == ADC_STATUS_SUCCESS) {

            adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);
            totalVolts = totalVolts + adcValue1MicroVolt[i];

            if (i % chunk == 0){
                avgVal = totalVolts/chunk;
                UART_PRINT("ADC channel 1 convert result (%d): %d uV\n\r", i, avgVal);
                totalVolts = 0;
            }
        }
        else {
            UART_PRINT("ADC channel 1 convert failed (%d)\n", i);
        }
        /* Original Example Code
         * for (i = 0; i < ADC_SAMPLE_COUNT; i++) {
            res = ADC_convert(adc, &adcValue1[i]);

            if (res == ADC_STATUS_SUCCESS) {

                adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);

                Display_printf(display, 0, 0, "ADC channel 1 convert result (%d): %d uV\n", i,
                                   adcValue1MicroVolt[i]);
            }
            else {
                Display_printf(display, 0, 0, "ADC channel 1 convert failed (%d)\n", i);
         }*/
    }

    ADC_close(adc);

    return (avgVal);
}

void CommandToAll(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * time = "set_time";

    UART_PRINT("\n\nCommand to all received!\n\n\r");

    if (queueElem->msgPtr) {
        if (strstr((const char *)queueElem->msgPtr + payloadOffset, time)) {
            UART_PRINT("Set time request received!\n\n\r");
            ParseTime(queueElem, topicOffset, payloadOffset);
            char * time = time_stamp();
            UART_PRINT("\n\nSystem Time updated to %s\n\r", time);
            free(time);
            getSystemTime();
        }
    }
}


void ParseCommand(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset)
{
    // expecting either send_config or scan.. call either accordingly

    // send_config (function should also be called by timer periodically
    // (build config update, then send over MQTT queue

    // scan(start_freq, stop_freq)
    char * scan = "scan";
    char * config = "send_config";

    UART_PRINT("\n\n\rParsing Message..\n\r");
    //UART_PRINT("Payload being parsed: %s\n\n\r", (char *)queueElem->msgPtr + payloadOffset);

    if (queueElem->msgPtr) {
        if (strstr((const char *)queueElem->msgPtr + payloadOffset, scan)) {
            UART_PRINT("Scan job received!\n\r");
            if (scan_flag) {
                UART_PRINT("Ignoring scan request, one is in progress..\n\r");
                return;
            } else {
                globalQueueElem = queueElem;
                globalPayloadOffset = payloadOffset;
                globalTopicOffset = topicOffset;
                scan_flag = 1;
            }
        } else if (strstr((const char*)queueElem->msgPtr + payloadOffset, config)) {
            UART_PRINT("Config requested!");
        } else {
            UART_PRINT("Unknown request from backend.");
        }
    } else {
        UART_PRINT("pointer to NULL :(");
    }

    return;
}


void ParseTime(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * hour_start;
    char * hour_stop;

    char * tmp;
    int offset = 0;

    char * time_start;

    char * day_start;
    char * day_stop;

    char * year_start;
    char * year_stop;

    char * timezone_start;
    char * timezone_stop;

    char * minute_start;
    char * minute_stop;

    char * second_start;
    char * second_stop;

    char * month_start;
    char * month_stop;

    char * day_month_start;
    char * day_month_stop;

    char * time = "\"time\"";


    /* find the time pointer    */
    time_start = strstr((const char *)queueElem->msgPtr + payloadOffset, time) + 9;
    if (time_start) {

        /*  get the day of the week */
        day_start = time_start;
        tmp = time_start;
        day_stop = strchr((const char*)tmp, ' ');
        *day_stop = 0;
        UART_PRINT("Found day of the week: %s\n\r", day_start);

        offset = day_stop - day_start + 1;

        /*  get the month */
        month_start = time_start + offset;
        tmp = time_start + offset;
        month_stop = strchr((const char*)tmp, ' ');
        *month_stop = 0;
        UART_PRINT("Found month: %s\n\r", month_start);

        offset = month_stop - day_start + 1;

        /*  get the day of the month */
        day_month_start = time_start + offset;
        tmp = time_start + offset;
        day_month_stop = strchr((const char*)tmp, ' ');
        *day_month_stop = 0;
        UART_PRINT("Found day of the month: %s\n\r", day_month_start);

        offset = day_month_stop - day_start + 1;

        /*  get the hour */
        hour_start = time_start + offset;
        tmp = time_start + offset;
        hour_stop = strchr((const char*)tmp, ':');
        *hour_stop = 0;
        UART_PRINT("Found hour: %s\n\r", hour_start);

        offset = hour_stop - day_start + 1;

        /*  get the minute */
        minute_start = time_start + offset;
        tmp = time_start + offset;
        minute_stop = strchr((const char*)tmp, ':');
        *minute_stop = 0;
        UART_PRINT("Found minute: %s\n\r", minute_start);

        offset = minute_stop - day_start + 1;

        /*  get the second */
        second_start = time_start + offset;
        tmp = time_start + offset;
        second_stop = strchr((const char*)tmp, ' ');
        *second_stop = 0;
        UART_PRINT("Found second: %s\n\r", second_start);

        offset = second_stop - day_start + 1;

        /*  get the timezone */
        timezone_start = time_start + offset;
        tmp = time_start + offset;
        timezone_stop = strchr((const char*)tmp, ' ');
        *timezone_stop = 0;
        UART_PRINT("Found time zone: %s\n\r", timezone_start);

        offset = timezone_stop - day_start + 1;

        /*  get the year */
        year_start = time_start + offset;
        tmp = time_start + offset;
        year_stop = strchr((const char*)tmp, '"');
        *year_stop = 0;
        UART_PRINT("Found year: %s\n\r", year_start);

        offset = year_stop - day_start + 1;

        dateTime.tm_day =   (_u32)atoi(day_month_start);          // Day of month (DD format) range 1-31
        dateTime.tm_year =  (_u32)atoi(year_start);        // Year (YYYY format)
        dateTime.tm_hour =  (_u32)atoi(hour_start);          // Hours in the range of 0-23
        dateTime.tm_min =   (_u32)atoi(minute_start);          // Minutes in the range of 0-59
        dateTime.tm_sec =   (_u32)atoi(second_start);          // Seconds in the range of  0-59


        if (!(strcmp(month_start, "January")))
            dateTime.tm_mon = 1;
        else if (!(strcmp(month_start, "February")))
            dateTime.tm_mon = 2;
        else if (!(strcmp(month_start, "March")))
            dateTime.tm_mon = 3;
        else if (!(strcmp(month_start, "April")))
            dateTime.tm_mon = 4;
        else if (!(strcmp(month_start, "May")))
            dateTime.tm_mon = 5;
        else if (!(strcmp(month_start, "June")))
            dateTime.tm_mon = 6;
        else if (!(strcmp(month_start, "July")))
            dateTime.tm_mon = 7;
        else if (!(strcmp(month_start, "August")))
            dateTime.tm_mon = 8;
        else if (!(strcmp(month_start, "September")))
            dateTime.tm_mon = 9;
        else if (!(strcmp(month_start, "October")))
            dateTime.tm_mon = 10;
        else if (!(strcmp(month_start, "November")))
            dateTime.tm_mon = 11;
        else if (!(strcmp(month_start, "December")))
            dateTime.tm_mon = 12;
        else {
            UART_PRINT("Invalid month given: %s\n\n\r", month_start);
            return;
        }

        //  set the system time
        setTime();


    } else {
        UART_PRINT("time not given! Exiting...\n\n\r");
        return;
    }

}



struct scan_vals initScan(struct msgQueue *queueElem, uint32_t topicOffset, uint32_t payloadOffset) {
    char * fmin_start;
    char * fmax_start;

    char * fmin_stop;
    char * fmax_stop;

    char * fmin = "fmin";
    char * fmax = "fmax";
    char * n_samples = "nsamples";

    char * n_samples_start;
    char * n_samples_stop;

    int fmin_val = 0;
    int fmax_val = 0;
    int n_samples_val = 0;

    struct scan_vals return_vals = {0};

    /*  check if the scan request has an n_samples value.. if so, extract and continue    */
    n_samples_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), n_samples) + 10);
    if (n_samples_start) {
        n_samples_stop = strchr((const char *)n_samples_start, ',');
        *n_samples_stop = 0;
        n_samples_val = atoi(n_samples_start) / 8;
        UART_PRINT("Found n_samples val: %s\t%d\n\r", n_samples_start, n_samples_val);
    } else {
        UART_PRINT("Scan received without n_samples value, canceling...\n\n\r");
        free(n_samples_start);
        return return_vals;
    }

    /* find the fmin pointer    */
    fmin_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), fmin) + 6);
    if (fmin_start) {
        fmin_stop = strchr((const char*)fmin_start, ',');
        *fmin_stop = 0;
        fmin_val = atoi(fmin_start);
        UART_PRINT("Found fmin val: %s\t%d\n\r", fmin_start, fmin_val);
    } else {
        UART_PRINT("FMIN not given! Exiting...\n\n\r");
        free(fmin_start);
        free(n_samples_start);
        return return_vals;
    }

    /* find the fmax pointer    */
    fmax_start = strdup(strstr((const char *)((char *)queueElem->msgPtr + payloadOffset), fmax) + 6);
    if (fmax_start) {
        fmax_stop = strchr((const char*)fmax_start, ',');
        *fmax_stop = 0;
        fmax_val = atoi(fmax_start);
        UART_PRINT("Found fmax val: %s\t%d\n\r", fmax_start, fmax_val);
    } else {
        UART_PRINT("FMAX not given! Exiting...\n\n\r");
        free(fmin_start);
        free(n_samples_start);
        free(fmax_start);
        return return_vals;
    }

    /*  convert ASCII scan values into ints */
    if (!(fmin_val)) {
        /*  given a zero value for frequency    */
        UART_PRINT("Zero freq value received!\n\n\r");
        return return_vals;
    }

     if (!(fmax_val)) {
         /*  given a zero value for frequency    */
         UART_PRINT("Zero freq value received!\n\n\r");
         return return_vals;
     }

     if (!(n_samples_val)) {
         /*  given a zero value for n_samples    */
         UART_PRINT("Zero freq value received!\n\n\r");
         return return_vals;
     }

    UART_PRINT("Scanning from:\n\rMin:\t%d MHz\n\rMax:\t%d MHz\n\rN_Samples:\t%d\n\r",
               fmin_val / 1000000, fmax_val / 1000000, n_samples_val);

    free(fmax_start);
    free(fmin_start);
    free(n_samples_start);

    return_vals.fmax = fmax_val;
    return_vals.fmin = fmin_val;
    return_vals.n_samples = n_samples_val;

    return(return_vals);
}

void ScanHelper(int fmin_val, int fmax_val, int n_samples_val) {
    uint8_t * data = Scan(fmin_val, fmax_val, n_samples_val);
    //char * demo_data = "demo";
    UART_PRINT("data returned\n\r");
    buildScanData(data, fmin_val, fmax_val, n_samples_val);
    SendScanData();
}

uint8_t * Scan(int fmin_val, int fmax_val, int n_samples) {
    //int actual_freq;
    float dispfreq;
    uint32_t adc_sample;

    /*  start timer */
    //TuneTimerConfigNStart();

    /*  calculate step size in MHz  */
    int step_size = (fmax_val - fmin_val + 1) / n_samples;
    UART_PRINT("Num samples: %d\n\r", n_samples);
    UART_PRINT("Step Size: %d\n\r", step_size);

    /*  scan through stuff  */
    int i = 0;
    int g = 0;
    for (i = fmin_val; i < fmax_val + 1; i++) {
        dispfreq = tune_PLL(i / 1000000, PLL_SPI);
        /*
         *
         while (!GPIO_read(Board_GPIO_12)) {
            dispfreq = tune_PLL(i / 1000000, PLL_SPI);
        }
        */
        (void)dispfreq;
        //while (!tune_flag){};
        UART_PRINT("Target set:\t%d\n\r", i / 1000000);
        adc_sample = ReadADC();
        data[g] = (uint8_t)(adc_sample / 1000);
        g++;
        //tune_flag = 0;
        i = i + step_size -1;              // 2 MHz steps until changed??
    }

    /*   int j;

    for (j = 0; j < n_samples; j++) {
        UART_PRINT("%02.02f\n\r", 20*log2f((float)data[j] / 1000));
    }
    */

    UART_PRINT("Done\n\r");

    //void TuneTimerDeinitStop();
    //UART_PRINT("timer off\n\r");

    return data;

}


void buildScanData(uint8_t * data, int fmin, int fmax, int n_samples) {

    /*  probably add a bunch of other appended payload vars here before sending using MSG_queue structure of button press   */

    char * str1 = "{\"software_version\": \"0.0.1.7\", \"timestamp\": \"";
    char * str2 = "\", \"gain\": 1, \"data\": \"";
    char * str3 = "\", \"short_name\": \"HoundDoggies\", \"altitude\": 2.0, \"uncertainty\": 100, \"batch\": 0, \"longitude\":  -86.099036999999996, \"sample_rate\": 2560000.0, \"mac_address\": \"9884e3f6643f\", \"latitude\": 41.539491000000001, \"center_frequency\": ";
    char * str4 = ", \"metadata\": {\"fmin\":";
    char * str5 = ", \"data_type\": \"periodogram\", \"fmax\": ";
    char * str6 = ", \"xcount\": ";
    char * str7 = ", \"xstop\": ";
    char * str8 = ", \"nfft\": ";
    char * str9 = ", \"xstart\": ";


    //char * demo = "6.005991131747623e-09, 1.5102760789010137e-08, 2.2146371086393924e-08, 1.698789560878992e-08, 8.377527434860303e-09, 8.596159654761953e-09, 1.5612401670304913e-08, 1.7083200989393155e-08, 1.6533841085112554e-08, 1.2824799978188182e-08, 1.2840626398823424e-08, 2.2542077837139032e-08, 2.17597143864778e-08, 1.3739492950269198e-08, 1.515223743340248e-08, 2.1124412433533475e-08, 8.703416214320775e-09, 9.761245528012853e-09, 8.039274981269685e-09, 6.4585044904093604e-09, 1.831344964842387e-08, 2.0519418813127655e-08, 2.28670349301937e-08, 2.381802874173875e-08, 1.680118776224668e-08, 8.558678461667735e-09, 2.005357045394623e-08, 2.6516670338352978e-08, 1.7740411143935347e-08, 1.245422920198992e-08, 1.1135251802600819e-08, 1.8554271093315888e-08, 1.8741304743402374e-08, 1.2629053936533511e-08, 1.0060284401105873e-08, 1.3101239554553378e-08, 1.4896999252130721e-08, 2.139735465510825e-08, 2.3594312712914835e-08, 1.4159440370727679e-08, 1.2398477374434699e-08, 1.3101607488339871e-08, 2.0487719741800037e-08, 1.8151790780576428e-08, 1.7207486659675396e-08, 3.0752957254198975e-08, 2.8329332961236436e-08, 1.4494646373362799e-08, 8.271504603711219e-09, 1.1605107305763273e-08, 1.3650835938247384e-08, 1.3243032833270727e-08, 1.7155600226542906e-08, 2.2621752649821276e-08, 2.806133645230524e-08, 2.5547738128207156e-08, 1.0438960845664642e-08, 1.7940116933924477e-08, 2.437126957933747e-08, 1.6745780932364935e-08, 3.560064155313113e-08, 2.6015707129641717e-08, 1.284042006359129e-08, 1.0334242232478277e-08, 1.54740293138865e-08, 3.0042024389424224e-08, 2.8205402487252393e-08, 1.0378773839486721e-08, 1.5436583729046304e-08, 2.4037142586043423e-08, 2.697522830390024e-08, 1.9089993809807856e-08, 2.2213430377278702e-08, 2.59296167813334e-08, 2.2090262195794497e-08, 1.715292484533063e-08, 2.9308514449226122e-08, 2.214026930102148e-08, 8.116759827065223e-09, 1.973958308289366e-08, 2.966873681183926e-08, 2.247702864204984e-08, 1.7724529097696665e-08, 2.5662242568034545e-08, 2.1690566376674787e-08, 1.9757887742750642e-08, 1.1709424836789613e-08, 1.0289999266249305e-08, 2.2107651011341374e-08, 2.366233018169962e-08, 2.1160723416941094e-08, 2.6996659786000032e-08, 2.17010827839153e-08, 1.4188333885222734e-08, 1.7756784643065192e-08, 2.4213365970858583e-08, 1.5971514876773696e-08, 1.7703463948182537e-08, 1.2827959366008931e-08, 1.4687583124950346e-08, 2.2671424533697657e-08, 2.5239802448714122e-08, 2.8226757601136358e-08, 1.7845428746341796e-08, 1.1137001879575544e-08, 1.2909858032699673e-08, 1.4280115204061098e-08, 2.1011315374060996e-08, 3.0486930767578065e-08, 9.81976644091114e-09, 1.611839995965741e-08, 2.84542339142745e-08, 3.394816228995965e-08, 1.7941149684016496e-08, 2.1933381784726813e-08, 2.787289030110399e-08, 2.3368154565049185e-08, 1.8707504369795873e-08, 3.8556866929780724e-08, 2.295340081687821e-08, 2.4430364148968847e-08, 2.9265032879092394e-08, 1.5847129825209492e-08, 3.716824121641396e-08, 3.004220993410587e-08, 2.6124719061214688e-08, 2.7730579975473195e-08, 3.570842221554715e-08, 4.911176446991316e-08, 4.3022648776444604e-08, 2.0947004368984493e-08, 2.2594846715188314e-08, 6.740592396938041e-08, 3.8319949387690694e-08, 1.7052291895614717e-08, 3.7313506181869965e-08, 2.1172880757012564e-08, 2.034525523792384e-08, 2.9237770384427332e-08, 2.2737614726057466e-08, 1.2537897616870976e-08, 1.5141124442381472e-08, 2.9490419801478065e-08, 3.195547646869825e-08, 1.954707421690285e-08, 2.7531805289755322e-08, 3.359784425040697e-08, 2.444849831964663e-08, 1.3540451965564295e-08, 1.5161633599017378e-08, 3.064020631150587e-08, 4.824904369422107e-08, 2.3858196625673675e-08, 2.6035614954356705e-08, 2.8405861449816547e-08, 2.52484389346107e-08, 3.048113504286092e-08, 1.6849640751837978e-08, 2.309996603218832e-08, 2.2913093949349744e-08, 1.695712790925187e-08, 2.694255006901135e-08, 2.6033056657880382e-08, 3.845650237078779e-08, 3.096207894223589e-08, 2.4251207732135866e-08, 2.1969207962661047e-08, 2.7175211204626826e-08, 4.4497266624812646e-08, 3.3164677537943914e-08, 3.393181039740046e-08, 2.3481229960050264e-08, 2.3433920948451356e-08, 2.9087647484731583e-08, 2.290726028714668e-08, 2.4546917075433407e-08, 2.307202272649787e-08, 2.1885420610910928e-08, 2.8454580841062425e-08, 2.654058950965728e-08, 1.8791503139019786e-08, 4.468676585093488e-08, 3.433758801195614e-08, 4.529195728454236e-08, 3.007314927169338e-08, 1.1894375807164939e-08, 1.0190437321881898e-08, 3.1972363706563045e-08, 3.678819040938902e-08, 3.545074765510802e-08, 3.915622457110442e-08, 2.956909440316668e-08, 2.3459112162688458e-08, 3.819475632695034e-08, 3.1014303330891233e-08, 2.3709139178815453e-08, 2.636939679576844e-08, 2.4454941675971504e-08, 3.4856723195157926e-08, 4.229785478001152e-08, 3.754839166461651e-08, 3.176802030733449e-08, 2.0383250711030083e-08, 1.5726793494745207e-08, 2.265205363772909e-08, 3.717406102277529e-08, 5.2429786757696956e-08, 4.500203758622136e-08, 3.028768209142983e-08, 1.369470863576149e-08, 9.680009771204272e-09, 2.3844342717295e-08, 1.92891299558839e-08, 1.6339238349914315e-08, 1.2365090975984797e-08, 1.708185872308799e-08, 2.7952909709820584e-08, 2.9410836824802877e-08, 1.9994214285405223e-08, 2.0945436066817522e-08, 2.4406295952921196e-08, 1.8237924058489114e-08, 2.6014564770654036e-08, 4.1850365835378245e-08, 2.891688087568678e-08, 2.0948341680192612e-08, 1.9841772831398413e-08, 2.516898880496636e-08, 2.540280251331357e-08, 2.929765836317903e-08, 5.8901918045803486e-08, 2.4053326366181093e-08, 8.226082336364104e-09, 9.867741368933481e-09, 1.7942657116222887e-08, 2.408526541310622e-08, 5.726280483532227e-08, 5.234641569020925e-08, 2.415840628465293e-08, 1.615628121752481e-08, 3.077365256183241e-08, 4.525252409093553e-08, 3.426967884146992e-08, 2.540688265530482e-08, 1.590778845815189e-08, 1.7173148864971572e-08, 1.5127914501886212e-08, 1.6124107951946134e-08, 1.786818274862204e-08, 2.4530391487493727e-08, 1.767165078556946e-08, 2.1040689147913634e-08, 1.612370951374772e-08, 9.859071077793167e-09, 2.2864374754168496e-08, 2.9019985625727084e-08, 2.995538044650424e-08, 1.5041390204228876e-08, 1.9183100565585077e-08, 1.4860607425686955e-08, 2.2234032919407154e-08, 1.9860353530236104e-08, 2.0767067169613998e-08, 4.192002296136145e-08, 4.0973169960647055e-08, 3.9434547205937387e-08, 3.2809085283559165e-08, 2.1228570485021686e-08, 2.4219571218215824e-08, 2.8420377667876344e-08, 2.1805031042325094e-08, 1.8826577664204867e-08, 2.4694353272858653e-08, 1.9548501871958763e-08, 1.977907342050124e-08, 1.9230866435708496e-08, 2.1435421897641377e-08, 3.071512961506755e-08, 4.915239709424586e-08, 9.588307872274435e-08, 1.9887560394058843e-08, 1.0154265665363309e-08, 1.7079847084253142e-08, 1.5128682675682606e-08, 1.2379607938387311e-08, 3.6095655747927194e-08, 3.845723868039588e-08, 2.3444076958470738e-08, 3.756196614742426e-08, 1.7001222794513185e-08, 1.9588873723670675e-08, 3.741116145227257e-08, 3.2853932336078595e-08, 2.8603168216763767e-08, 2.3925528242589873e-08, 1.2980459778163961e-08, 1.6498820508123163e-08, 3.313225292606027e-08, 4.0190665980143726e-08, 3.061376921947191e-08, 2.51834658234187e-08, 2.9863521177836094e-08, 3.0114646055771496e-08, 7.221058877341661e-08, 5.390966625525901e-08, 2.3324047174051727e-08, 2.7164990124293514e-08, 2.4300249358472307e-08, 1.4738508960794716e-08, 2.455433854016051e-08, 3.802625687114874e-08, 2.9533023747117835e-08, 2.060246662036126e-08, 2.8948412365782865e-08, 2.9490761019461256e-08, 1.2982137866193262e-08, 2.4852258779905142e-08, 2.1306860560556808e-08, 3.288938953215975e-08, 1.593324966960801e-08, 1.8640382591433532e-08, 1.6092230266085256e-08, 1.279708577626733e-08, 3.153477558186123e-08, 2.1818143939102983e-08, 2.9104283102858395e-08, 2.3554964326417576e-08, 3.685956109101807e-08, 3.483503355453994e-08, 3.2748830071109716e-08, 3.2745182221068504e-08, 2.836952818532475e-08, 2.512513694900031e-08, 2.0233208836731263e-08, 1.2757025803809992e-08, 1.818270625550972e-08, 1.9811571934068687e-08, 1.9835952940256e-08, 2.1046361523101172e-08, 2.7280116139939065e-08, 1.2625713815488272e-08, 1.9050278808717767e-08, 2.4213616282926935e-08, 1.8305039693262983e-08, 2.0832774618453688e-08, 1.89731686821312e-08, 1.4443041298531512e-08, 1.8778149749756268e-08, 2.5263137083642263e-08, 3.33307770943669e-08, 2.8934972346271494e-08, 3.095745358385224e-08, 3.046153544372927e-08, 2.1272922395909426e-08, 2.3442128591876183e-08, 1.9217404956561774e-08, 2.337931591695676e-08, 2.944042775664652e-08, 2.3119830924287477e-08, 1.6713155530963564e-08, 2.9018820206290717e-08, 2.386067192745724e-08, 1.4686734249807685e-08, 1.793215640575906e-08, 2.4087803589072442e-08, 2.2893421482355174e-08, 1.9617637551875265e-08, 2.9748847409497822e-08, 3.2408063325957156e-08, 3.7818588751807846e-08, 3.36796116214962e-08, 2.0368190717575332e-08, 1.6391648659176778e-08, 2.649580565829936e-08, 2.7186474285422494e-08, 2.2130587774040706e-08, 2.0880798216914747e-08, 3.566507786517221e-08, 3.038649631460697e-08, 1.2721884639259397e-08, 3.7385135953701494e-08, 3.2415149054176314e-08, 2.6571051319243406e-08, 2.621711094484667e-08, 4.78541295435651e-08, 2.847075417838806e-08, 2.045012020842774e-08, 6.6644602130144375e-09, 8.402191107356699e-09, 2.2074888217424148e-08, 3.373201910294933e-08, 2.7462330698917723e-08, 2.2888933752085533e-08, 3.327609079236336e-08, 3.839535620386471e-08, 2.3730264235299826e-08, 3.0547924135849234e-08, 3.787688508956622e-08, 3.024880261584901e-08, 1.3905642331949294e-08, 2.3031798213525143e-08, 3.779540866282404e-08, 1.985254809141297e-08, 1.0680442654030492e-08, 2.167116744577222e-08, 2.2405016814635267e-08, 3.3832580464870595e-08, 3.597462410886766e-08, 2.5188830781103235e-08, 3.92660623285251e-08, 4.2452696527415e-08, 3.1633013484824054e-08, 1.7310277394305735e-08, 1.7066566120609462e-08, 2.5091158056486546e-08, 1.3685105278764671e-08, 1.7927504956091818e-08, 4.0561068086184474e-08, 3.900364359469587e-08, 1.7869643669707155e-08, 2.6813948819108218e-08, 2.6731250134732286e-08, 1.7807661560974037e-08, 2.269054252780816e-08, 5.680138406822599e-08, 5.0017232269985775e-08, 4.0952029845001194e-08, 4.0812427541499986e-08, 5.5649043468266026e-08, 6.737498558823217e-08, 4.5351937652393535e-08, 3.2281907391136076e-08, 6.827873446658843e-08, 8.316429964841374e-08, 1.0639468620229016e-07, 7.926437431315338e-08, 1.1515349608688852e-07, 1.675270260903073e-07, 1.4583546241640168e-07, 1.0676651147216005e-07, 1.8289303865426815e-07, 1.62306394860428e-07, 3.404957625023067e-07, 2.7098855314350163e-07, 2.0960478642161885e-07, 2.747415115542859e-07, 4.1432625744455683e-07, 3.9855200307161536e-07, 2.9227404305146663e-07, 4.325827039300736e-07, 1.1488629825955956e-06, 7.312624775886885e-07, 1.0703785590865649e-06, 6.429359514308732e-07, 6.53846997944309e-07, 1.1279338715524097e-06, 8.336422851036289e-07, 8.355057562215791e-07, 5.40679748607422e-07, 9.441777153846347e-07, 1.0303580913387565e-06, 1.4404665532134304e-06, 9.154713670224347e-07, 6.244071889871983e-07, 4.205587046103738e-07, 3.4159466804648805e-07, 5.275865466813863e-07, 4.412771194102047e-07, 3.4513772816990856e-07, 2.645884207334713e-07, 5.088540688972722e-07, 5.360591456644186e-07, 2.867830122263645e-07, 2.148020966146479e-07, 1.7635013945091875e-07, 1.2286594922386552e-07, 1.0309809730873882e-07, 6.78900384863946e-08, 6.521783097977558e-08, 6.084413323373063e-08, 7.75175639473887e-08, 7.769450065698877e-08, 7.074605684228564e-08, 6.947953244697208e-08, 3.928367334966182e-08, 1.754180251835265e-08, 3.312897630625946e-08, 3.115630474741834e-08, 2.8188719222929197e-08, 3.5343870999447924e-08, 3.880267419028494e-08, 2.936118097924748e-08, 2.6414463821809847e-08, 1.7675870509447563e-08, 1.3433725987934982e-08, 2.212199049016633e-08, 2.530735877686042e-08, 2.611484951735005e-08, 2.5702405236909743e-08, 1.2275933440274579e-08, 2.3935089196994635e-08, 3.6448580314371086e-08, 3.619101463201199e-08, 2.8038953584866336e-08, 1.0365123919901872e-08, 2.607355981348725e-08, 4.1909852127553846e-08, 4.555109441330531e-08, 4.4290777996135655e-08, 2.23305963282026e-08, 3.349669304131722e-08, 3.858129983904251e-08, 1.618643211720566e-08, 1.8615894910654702e-08, 2.0965183645495008e-08, 1.750870270521792e-08, 1.654881752825234e-08, 2.184168158512444e-08, 1.904022901758206e-08, 2.929950008311805e-08, 2.9269520001723625e-08, 3.534376638248206e-08, 2.3335357232847708e-08, 2.3532433936897936e-08, 1.450520763246564e-08, 2.4858141667808848e-08, 2.7707175883400605e-08, 2.2288594126913512e-08, 2.4845928072706095e-08, 3.4574165519465565e-08, 2.7099531669829407e-08, 3.9095132962908566e-08, 3.56083273316311e-08, 2.3003329167447945e-08, 3.219491574011261e-08, 1.9045929602377087e-08, 2.6853530515279086e-08, 2.9025733576373e-08, 3.548916870743962e-08, 4.4151421380012294e-08, 2.022648050532757e-08, 3.6216217702274955e-08, 2.5703503563796952e-08, 2.5221352298466188e-08, 2.3144914908232268e-08, 1.8681506813606313e-08, 2.536041370836486e-08, 3.155690709813266e-08, 3.633723995769977e-08, 4.381044855165758e-08, 2.5591874993875702e-08, 1.7040108015171002e-08, 1.4276970514177331e-08, 1.7095536518541193e-08, 2.929561467486128e-08, 2.9023949208755518e-08, 2.456270879291676e-08, 3.6094878117815144e-08, 4.245897319992218e-08, 3.3110680994199544e-08, 2.029935076325449e-08, 2.441504366515025e-08, 1.662448599670503e-08, 1.4032925385823885e-08, 1.6150527864549114e-08, 2.259001761126459e-08, 2.9291865362059788e-08, 4.825463065621585e-08, 3.48472479180154e-08, 2.3767726649676547e-08, 2.324191114910958e-08, 2.9446472621574587e-08, 2.3940609552330038e-08, 3.142882730205169e-08, 2.4576737839592767e-08, 1.617479119142088e-08, 1.2958181693765799e-08, 1.4624175400497429e-08, 2.3160194335069226e-08, 2.5335679285259057e-08, 2.4724321621885333e-08, 1.866640121100838e-08, 2.2337020712557613e-08, 1.8722067033390296e-08, 1.7726029707146973e-08, 1.0802983492394748e-08, 2.6167392764555868e-08, 2.1692617148221542e-08, 2.08056369315395e-08, 2.2086713854372133e-08, 1.3784699157335492e-08, 5.3258680559941975e-09, 2.563814700747999e-08, 1.7742873670330203e-08, 2.0202487984756393e-08, 1.8946221320801755e-08, 2.4419930857430264e-08, 1.3709851283467028e-08, 2.484089804145327e-08, 4.876731883225851e-08, 3.07136496960475e-08, 2.9032752761344513e-08, 2.550544882218338e-08, 4.817238583380812e-08, 5.219060789430857e-08, 3.6388046114099516e-08, 3.533812018565165e-08, 1.7354533517581778e-08, 9.745768446101718e-09, 1.9451381101157372e-08, 1.8454365181606043e-08, 2.6588099852385194e-08, 3.2198954559743434e-08, 1.703690601089534e-08, 1.740375094786953e-08, 3.267792970848148e-08, 1.9355397459156653e-08, 2.9931606727370875e-08, 4.286148037365279e-08, 3.02553757862148e-08, 1.0850462422192177e-08, 1.5981805282578947e-08, 2.0551161119604915e-08, 3.815271015720274e-08, 4.7086360933557526e-08, 4.149805778271254e-08, 3.575611244219682e-08, 3.990462692146328e-08, 2.51398852407639e-08, 2.5185552723768396e-08, 2.3181924277827986e-08, 4.208014581920257e-08, 3.242663808378309e-08, 2.6864384137268007e-08, 4.0650195172194694e-08, 3.398639669251199e-08, 1.8336476246015114e-08, 2.231833479894216e-08, 2.477073754743294e-08, 1.529663728905647e-08, 2.6873069095017093e-08, 3.0150870766022274e-08, 1.6223057004762665e-08, 2.1226037058835695e-08, 3.3516297225303616e-08, 5.708976937943902e-08, 3.881887326023387e-08, 3.1133327788870894e-08, 3.7567487460588215e-08, 2.923882746004822e-08, 3.446891784334902e-08, 4.5333502081202886e-08, 3.89252769442849e-08, 3.0128463055617827e-08, 1.5564520714486055e-08, 1.8568786074941164e-08, 2.1061908782251117e-08, 1.8853879844990015e-08, 2.218338259045662e-08, 2.04377770093279e-08, 2.2545890730613976e-08, 1.5491709513244772e-08, 2.288150654472265e-08, 2.129502968898837e-08, 1.601669439291946e-08, 1.2227830044136906e-08, 2.2715634384231475e-08, 3.6986507309706206e-08, 2.0529890079938665e-08, 1.9082328379989003e-08, 3.110137680107037e-08, 4.011071709099119e-08, 3.732155925731798e-08, 2.285098029440265e-08, 1.516357264890414e-08, 2.3386329838363235e-08, 9.466430065907159e-09, 2.4523916119898094e-08, 2.842364396932633e-08, 1.0622826232809963e-08, 8.211447196263091e-09, 2.110434560870835e-08, 2.5990482072397654e-08, 1.6715529683201086e-08, 2.5713203946200413e-08, 4.4014129954991986e-08, 2.4890876031688844e-08, 7.602443424863673e-09, 1.64237791820355e-08, 2.0398999485587017e-08, 1.5328558963860915e-08, 6.534941865212107e-09, 1.9494478743728706e-08, 3.767740032945621e-08, 3.144412815058918e-08, 1.5665754610408113e-08, 2.19634307450665e-08, 3.0252493900748746e-08, 1.6642530359691136e-08, 1.9280925744029885e-08, 3.085190902037145e-08, 2.784271885458701e-08, 1.9137678900834524e-08, 1.556685718995311e-08, 1.6201393667687343e-08, 1.683994586363967e-08, 2.2234590818092328e-08, 2.1274164369516775e-08, 1.078082239786802e-08, 1.3909244534014859e-08, 2.4528461020892813e-08, 2.8288236039710804e-08, 2.3666379439923813e-08, 2.8078788527579274e-08, 4.597877349460177e-08, 3.6149168322247745e-08, 3.144178068710186e-08, 5.0265254091207974e-08, 2.982989149451732e-08, 3.1993988152211514e-08, 3.008196471834179e-08, 2.1463219053866066e-08, 2.3852951940363533e-08, 1.3454734817039249e-08, 2.983864542032787e-08, 3.538631800725684e-08, 2.2776052413035233e-08, 2.5790921458215106e-08, 1.4113101024340279e-08, 1.3914314026488158e-08, 1.5702249240755596e-08, 3.2339906405344e-08, 3.062034038664141e-08, 1.8535657388690133e-08, 3.342607112671764e-08, 3.0344540691833605e-08, 1.508925468902082e-08, 1.784390129907692e-08, 2.321686041288098e-08, 1.4392624699816586e-08, 2.138813245232798e-08, 2.003330607962028e-08, 2.7248440118721876e-08, 2.5758008729909504e-08, 2.0574107181077708e-08, 1.490120003842081e-08, 1.650611619460726e-08, 2.830611996089236e-08, 3.393428846710985e-08, 3.0366981618357855e-08, 2.3848062352109028e-08, 8.360570776286929e-09, 2.242222924185067e-08, 2.1546837398553637e-08, 2.0549551131556606e-08, 1.5725754345723213e-08, 5.9945235221101585e-09, 7.835214015361957e-09, 1.2460606500701719e-08, 2.2176732998877845e-08, 2.1015411979582617e-08, 2.2363627339176058e-08, 2.9876442216488047e-08, 8.044789961524441e-09, 2.0508793893075338e-08, 2.3144201911910473e-08, 2.2095746225935755e-08, 2.1560824788084838e-08, 2.3080326611057536e-08, 3.9789838574267535e-08, 5.579465829622211e-08, 4.348813628856404e-08, 2.6448521060272486e-08, 2.425839157034494e-08, 2.1991552553067594e-08, 2.3888073441760076e-08, 1.4018679033741117e-08, 1.4336898348669814e-08, 2.2940791135526072e-08, 1.7289820709501422e-08, 9.14477685260187e-09, 1.3005172123160232e-08, 1.3966461278871318e-08, 2.1869894307427785e-08, 2.1919975756716162e-08, 2.510938821296033e-08, 1.8856307700017573e-08, 3.5791265160703095e-08, 3.217505462844457e-08, 1.4206084712472984e-08, 1.8127855208811815e-08, 1.9225111211057513e-08, 1.704453726527238e-08, 2.988351645674375e-08, 2.9301090072923592e-08, 3.2603406249731546e-08, 3.269746551065445e-08, 3.139694517935422e-08, 2.2598902081483767e-08, 3.5530894884894435e-08, 3.0772841379944304e-08, 2.0380750482517634e-08, 1.9173058359004672e-08, 3.388215620708014e-08, 3.0561702421053815e-08, 2.9601451286567243e-08, 2.4071814994531164e-08, 3.047524272516985e-08, 3.104820605456469e-08, 2.529406474024337e-08, 3.329688862461162e-08, 2.382572860775165e-08, 1.7889784933863225e-08, 1.9232638945948728e-08, 2.458573373714006e-08, 1.752561832333799e-08, 2.6171559751658784e-08, 3.538103740738052e-08, 3.8820303276234524e-08, 2.8079469433690554e-08, 3.213525671102744e-08, 2.675685311809038e-08, 1.316163097769869e-08, 1.6234954401300134e-08, 1.6762640909391338e-08, 2.2622878530663456e-08, 3.5185034047474785e-08, 5.5642514192736055e-08, 2.876765616587453e-08, 1.8243694402200578e-08, 2.7718971254949627e-08, 2.591107084147069e-08, 3.090165658182368e-08, 8.894695227359945e-09, 1.1648318092350404e-08, 1.322516462766557e-08, 1.6896067147090914e-08, 1.7780664093763286e-08, 2.542222782724882e-08, 2.9525484787842695e-08, 1.1321603898886888e-08, 1.2668830359209684e-08, 2.035361094442069e-08, 2.354347004688171e-08, 1.69096802726715e-08, 1.5618551617937658e-08, 1.834772190807107e-08, 2.192047910085099e-08, 6.3206912218343264e-09, 8.06261638015157e-09, 1.2402331719197868e-08, 1.6324065531410887e-08, 2.2354228482554183e-08, 2.693656836118679e-08, 2.4615883797428232e-08, 1.2303349780972637e-08, 1.3308986149194071e-08, 1.8509480263693736e-08, 2.6633051158147365e-08, 3.2214498463305694e-08, 5.6927118906246395e-08, 3.6772408895206766e-08, 2.489184488604696e-08, 2.489970363322851e-08, 1.9326042332075337e-08, 3.469615131636644e-08, 3.632350879473353e-08, 1.3769505023579693e-08, 1.9765471380593658e-08, 3.875530278620124e-08, 3.2929705773772584e-08, 2.1862598661540408e-08, 2.1112479826692594e-08, 2.8560344865551724e-08, 3.230360083309517e-08, 2.8314082044737644e-08, 3.11360275902454e-08, 4.267647554559533e-08, 4.642431243139956e-08, 2.5709378701824236e-08, 2.4649082203042712e-08, 2.260218048289168e-08, 3.4873917778695145e-08, 3.207257681260276e-08, 2.1520709834216344e-08, 3.281817224665008e-08, 4.9380456413088654e-08, 3.1339620414351444e-08, 2.9736989047084828e-08, 2.7376100839611134e-08, 4.3557095498040263e-08, 4.714966450917901e-08, 5.0419479839638595e-08, 2.7294258112018614e-08, 1.272489489574085e-08, 1.7012788221436697e-08, 4.419119087596255e-09, 9.357284366898472e-09, 2.3156857212250268e-08, 1.9467582042945075e-08, 1.641451273168033e-08, 1.0083398996874584e-08, 1.850066613016454e-08, 2.7286686725013153e-08, 1.3525981428119373e-08, 2.2682475860369865e-08, 2.4942205769527717e-08, 1.8307023548560808e-08, 3.104226634937466e-08, 3.020575413667896e-08, 3.209130258822749e-08, 2.3878614852009912e-08, 1.6567679157818368e-08, 1.3643517937502747e-08, 1.7002932263322117e-08, 1.3419103541814125e-08, 2.3108794734534957e-08, 2.096374629874476e-08, 2.527274362211277e-08, 1.8905796603726762e-08, 1.6445457870897296e-08, 2.0366699049452473e-08, 2.507357431537272e-08, 1.4832762219368224e-08, 2.1566257507940206e-08, 2.7988475066262072e-08, 1.005846586683825e-08, 5.206015822369566e-09, 1.4705917629181765e-08, 2.213811587795633e-08, 1.79054392224621e-08, 2.480515030044546e-08, 2.3853261877688908e-08, 1.572862021641671e-08, 1.6978826808204603e-08, 2.2379206501185154e-08, 1.7429822990632088e-08, 1.015661531435373e-08, 9.152415892221728e-09, 9.586711883187464e-09, 1.1054353198438665e-08, 2.5142174719304813e-08, 1.4544919680170477e-08, 9.74992390749481e-09, 1.4377187392796705e-08, 1.070125286154347e-08, 1.1477776660081395e-08, 1.283454768360121e-08, 1.3000434599352874e-08, 8.996275866113357e-09, 1.141430609744434e-08, 2.0097279370697065e-08, 1.4603331842824108e-08, 1.837882953444163e-08, 2.15346091793197e-08, 1.7804604722694796e-08, 2.0296487939773726e-08, 2.0092237719086134e-08, 1.0859043349702868e-08, 6.9121151182222235e-09, 1.4024755894347509e-08, 1.6907689002163562e-08, 1.4289679363851093e-08, 1.6364375230407375e-08, 1.2290695697906493e-08, 8.101368118146282e-09, 6.9230879657702795e-09, 1.3276047907388213e-08, 1.992726746707269e-08, 6.89981677408223e-09, 1.9357674905881473e-08, 2.118589361892499e-08, 2.693439062288206e-08, 1.9990839110534926e-08, 1.9651067861483412e-08, 1.6034866866021015e-08, 1.344537134266464e-08, 2.2789114133730676e-08, 1.3399242223425167e-08, 1.5333711062701237e-08, 9.2016653139079e-09, 2.7403834042463884e-08, 3.5873524814174344e-08, 2.2020172962938993e-08, 1.221440039226437e-08, 7.490674164202577e-09, 1.2548663908054733e-08, 2.288714585934608e-08, 3.0322158753738173e-08, 2.6265678501398306e-08, 2.0486006303536302e-08, 2.3985583617582594e-08, 1.9135650851594337e-08, 1.6951266234151736e-08, 1.5425643564724513e-08, 9.287129037684567e-09, 1.5201714838412585e-08, 8.464114751603569e-09, 1.9780396019774768e-08, 1.8510645359272137e-08, 1.229975116952122e-08, 1.8388186129447273e-08, 2.057690734942735e-08, 2.0059245724878276e-08, 1.2758928226642808e-08, 1.0779722756674227e-08, 5.340470402147806e-09, 2.1735921316234442e-08, 1.3723356557041599e-08, 1.6264558250984245e-08, 1.976956945687032e-08, 1.918393114874948e-08, 1.557704874368907e-08, 1.32241185938853e-08, 1.829192367626584e-08, 2.5450893609774313e-08, 1.4072111814391456e-08, 1.0685159564345388e-08, 9.514786596787953e-09, 2.0778769638326983e-08, 2.5480413607624703e-08, 1.751540860150307e-08, 9.327799455879883e-09, 6.999420727557586e-09";

    UART_PRINT("Definitions done\n\r");

    char * temp_time = time_stamp();
    sprintf(str_tmp, "%s%s%s", str1, temp_time, str2);
    strcat(payload, str_tmp);
    free(temp_time);
    float temp = 0;
    float pwr = 0;

    if (!DEMO) {
        int i;
        for (i = 0; i < n_samples; i++){
            if (i == n_samples - 1) {
                pwr = pow(((float)data[i] / 1000000), 2) / 50;
                sprintf(str_tmp, "%2.1f",  20*log2f(pwr/.001));
                strcat(payload, str_tmp);
            } else {
                UART_PRINT("data = %d\n\r", data[i]);
                pwr = pow(((float)data[i] / 1000000), 2) / 50;
                temp = 20*log2f(pwr/.001);
                UART_PRINT("temp = %f\n\r", temp);
                sprintf(str_tmp, "%2.1f, ", temp);
                strcat(payload, str_tmp);
            }
            UART_PRINT("For %d\t%s\n\r", i, str_tmp);
        }
    } else {
       // strcat(payload, demo);
    }

    UART_PRINT("Payload with data: %s\n\n\r", payload);

    sprintf(str_tmp, "%s%d%s%d%s%d%s%d%s%d%s%d%s%d}}", str3, fmax, str4, fmin, str5, fmax,
            str6, n_samples, str7, fmax, str8, n_samples, str9, fmin);
    strcat(payload, str_tmp);

    UART_PRINT("PAYLOAD:\n\n\r%s\n\n\r", payload);
}


void SendScanData() {
    /* Send Announce packet to RadioHound                                     */
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    queueElement.event = PAYLOAD_SEND;
    queueElement.msgPtr = NULL;

    /* write message indicating publish message, skipping queue               */
    UART_PRINT("About to send payload to queue...\n\r");

    UART_PRINT("Size of payload: %d\n\r", strlen((char*)payload));


    /* Send Announce packet to RadioHound                                     */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full!  Unclogging...\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), 0);
        MQTT_SendMsgToQueue(&queueElement);
    }

    /*  publishing to RadioHound Backend    */
    UART_PRINT("Published data to queue\n\r");
}





//*****************************************************************************
//
//! Application startup display on UART
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
static void DisplayBanner(char * AppName)
{
    UART_PRINT("\n\n\n\r");
    UART_PRINT("\t\t *************************************************\n\r");
    UART_PRINT("\t\t    CC3220 %s Application       \n\r", AppName);
    UART_PRINT("\t\t *************************************************\n\r");
    UART_PRINT("\n\n\n\r");
}

void *MqttClientThread(void * pvParameters)
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    MQTTClient_run((NetAppHandle_t)pvParameters);

    queueElement.event  = LOCAL_CLIENT_DISCONNECTION;
    queueElement.msgPtr = NULL;

    /* write message indicating disconnect Broker message.                    */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }

    pthread_exit(0);

    return NULL;
}


//*****************************************************************************
//
//! Task implementing MQTT Server plus client bridge
//!
//! This function
//!    1. Initializes network driver and connects to the default AP
//!    2. Initializes the mqtt client ans server libraries and set up MQTT
//!       with the remote broker.
//!    3. set up the button events and their callbacks(for publishing)
//!    4. handles the callback signals
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
void * MqttClient(void *pvParameters)
{

    struct msgQueue queueElemRecv;
    long lRetVal = -1;
    char *tmpBuff;

    /* Initializing Client and Subscribing to the Broker.                     */
    if (gApConnectionState >= 0)
    {
        lRetVal = MqttClient_start();
        if (lRetVal == -1)
        {
            UART_PRINT("MQTT Client lib initialization failed\n\r");
            pthread_exit(0);
            return NULL;
        }
    }

    /* handling the signals from various callbacks including the push button  */
    /* prompting the client to publish a msg on PUB_TOPIC OR msg received by  */
    /* the server on enrolled topic(for which the on-board client ha enrolled)*/
    /* from a local client(will be published to the remote broker by the      */
    /* client) OR msg received by the client from the remote broker (need to  */
    /* be sent to the server to see if any local client has subscribed on the */
    /* same topic).                                                           */
    for (;;)
    {
        /* waiting for signals                                                */
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);

        switch (queueElemRecv.event)
        {
            case PUBLISH_PUSH_BUTTON_PRESSED:

                /* send publish message                                       */
                lRetVal = MQTTClient_publish(gMqttClient, (char*) publish_topic, strlen((char*)publish_topic), (char*)publish_data, strlen((char*) publish_data), MQTT_QOS_2 | ((RETAIN_ENABLE)?MQTT_PUBLISH_RETAIN:0) );

                UART_PRINT("\n\r CC3200 Publishes the following message \n\r");
                UART_PRINT("Topic: %s\n\r", publish_topic);
                UART_PRINT("Data: %s\n\r", publish_data);

                UART_PRINT("return val: %d\n\r", lRetVal);


                /* Clear and enable again the SW2 interrupt */
                GPIO_clearInt(Board_BUTTON0); // SW2
                GPIO_enableInt(Board_BUTTON0); // SW2

                break;

                /* msg received by client from remote broker (on a topic      */
                /* subscribed by local client)                                */
            case PAYLOAD_SEND:

                /*  publishing to RadioHound Backend    */
                UART_PRINT("Received Payload in MQTT Thread\n\r");

                UART_PRINT("Size of payload mqtt: %d\n\r", strlen((char*)payload));

                /* send publish message                                       */
                lRetVal = MQTTClient_publish(gMqttClient, (char*) publish_payload, strlen((char*)publish_payload), (char*)payload, strlen((char*) payload), MQTT_QOS_2 | ((RETAIN_ENABLE)?MQTT_PUBLISH_RETAIN:0) );

                UART_PRINT("return val: %d\n\r", lRetVal);

                UART_PRINT("\n\r CC3200 Publishes the following payload \n\r");
                UART_PRINT("Topic: %s\n\r", publish_payload);
                UART_PRINT("Data: %s\n\r", payload);

                break;

                /* msg received by client from remote broker (on a topic      */
                /* subscribed by local client)                                */
            case MSG_RECV_BY_CLIENT:
                tmpBuff = (char *) ((char *) queueElemRecv.msgPtr + 12);
                if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC1, queueElemRecv.topLen) == 0)
                {
                    /* for manipulating payload data    */
                    uint32_t topicOffset = 0;
                    uint32_t payloadOffset = 0;

                    /*  toggle LED to indicate received message */
                    GPIO_toggle(Board_LED0);

                    /*  set the topic and payload offsets   */
                    topicOffset = sizeof(struct publishMsgHeader);
                    payloadOffset = sizeof(struct publishMsgHeader) + queueElemRecv.topLen + 1;

                    ParseCommand(&queueElemRecv, topicOffset, payloadOffset);        // either send config or scan
                }
                /*  this is where handling of other events will occur   */
                else if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC0, queueElemRecv.topLen) == 0)
                {
                    /*  this is a command to all, hopefully to set the current time */

                    /* for manipulating payload data    */
                    uint32_t topicOffset = 0;
                    uint32_t payloadOffset = 0;

                    GPIO_toggle(Board_LED1);

                    /*  set the topic and payload offsets   */
                    topicOffset = sizeof(struct publishMsgHeader);
                    payloadOffset = sizeof(struct publishMsgHeader) + queueElemRecv.topLen + 1;

                    CommandToAll(&queueElemRecv, topicOffset, payloadOffset);    // set system time
                    free(queueElemRecv.msgPtr);
                } else {
                    free(queueElemRecv.msgPtr);
                }
                /*
                else if (strncmp(tmpBuff, SUBSCRIPTION_TOPIC2, queueElemRecv.topLen) == 0)
                {
                    GPIO_toggle(Board_LED2);
                }
                */

                break;

                /* On-board client disconnected from remote broker, only      */
                /* local MQTT network will work                               */
            case LOCAL_CLIENT_DISCONNECTION:
                UART_PRINT("\n\rOn-board Client Disconnected\n\r\r\n");
                gUiConnFlag = 0;
                break;

                /* Push button for full restart check                         */
            case DISC_PUSH_BUTTON_PRESSED:
                gResetApplication = true;
                break;
                
            case THREAD_TERMINATE_REQ:
                gUiConnFlag = 0;
                pthread_exit(0);
                return NULL;
                
            default:
                sleep(1);
                break;
        }
    }
}

//*****************************************************************************
//
//! This function connect the MQTT device to an AP with the SSID which was 
//! configured in SSID_NAME definition which can be found in Network_if.h file, 
//! if the device can't connect to to this AP a request from the user for other 
//! SSID will appear.
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
int32_t Mqtt_IF_Connect()
{
    int32_t lRetVal;
    char SSID_Remote_Name[32];
    int8_t Str_Length;

    memset(SSID_Remote_Name, '\0', sizeof(SSID_Remote_Name));
    Str_Length = strlen(SSID_NAME);

    if (Str_Length)
    {
        /* Copy the Default SSID to the local variable                        */
        strncpy(SSID_Remote_Name, SSID_NAME, Str_Length);
    }

    /* Display Application Banner                                             */
    DisplayBanner(APPLICATION_NAME);

    GPIO_write(Board_LED0, Board_LED_OFF);
    GPIO_write(Board_LED1, Board_LED_OFF);
    GPIO_write(Board_LED2, Board_LED_OFF);

    /* Reset The state of the machine                                         */
    Network_IF_ResetMCUStateMachine();

    /* Start the driver                                                       */
    lRetVal = Network_IF_InitDriver(ROLE_STA);
    if (lRetVal < 0)
    {
        UART_PRINT("Failed to start SimpleLink Device\n\r", lRetVal);
        return -1;
    }

    /* switch on Green LED to indicate Simplelink is properly up.             */
    GPIO_write(Board_LED2, Board_LED_ON);

    /* Start Timer to blink Red LED till AP connection                        */
    LedTimerConfigNStart();

    /* Initialize AP security params                                          */
    SecurityParams.Key = (signed char *) SECURITY_KEY;
    SecurityParams.KeyLen = strlen(SECURITY_KEY);
    SecurityParams.Type = SECURITY_TYPE;

    /* Connect to the Access Point                                            */
    lRetVal = Network_IF_ConnectAP(SSID_Remote_Name, SecurityParams);
    if (lRetVal < 0)
    {
        UART_PRINT("Connection to an AP failed\n\r");
        return -1;
    }

    /* Disable the LED blinking Timer as Device is connected to AP.           */
    LedTimerDeinitStop();

    /* Switch ON RED LED to indicate that Device acquired an IP.              */
    GPIO_write(Board_LED0, Board_LED_ON);

    sleep(1);

    /* Enable ANNOUNCE timer to send packets to RadioHound backend every 10 sec */
    AnnounceTimerConfigNStart();

    GPIO_write(Board_LED0, Board_LED_OFF);
    GPIO_write(Board_LED1, Board_LED_OFF);
    GPIO_write(Board_LED2, Board_LED_OFF);

    return 0;
}

//*****************************************************************************
//!
//! MQTT Start - Initialize and create all the items required to run the MQTT
//! protocol
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************
void Mqtt_start()
{
    int32_t threadArg = 100;
    pthread_attr_t pAttrs;
    struct sched_param priParam;
    int32_t retc = 0;
    mq_attr attr;
    unsigned mode = 0;

    /* sync object for inter thread communication                             */
    attr.mq_maxmsg = 8;
    attr.mq_msgsize = sizeof(struct msgQueue);
    g_PBQueue = mq_open("g_PBQueue", O_CREAT, mode, &attr);

    if (g_PBQueue == NULL)
    {
        UART_PRINT("MQTT Message Queue create fail\n\r");
        gInitState &= ~MQTT_INIT_STATE;
        return;
    }

    /* Set priority and stack size attributes                                 */
    pthread_attr_init(&pAttrs);
    priParam.sched_priority = 2;
    retc = pthread_attr_setschedparam(&pAttrs, &priParam);
    retc |= pthread_attr_setstacksize(&pAttrs, MQTTTHREADSIZE);
    retc |= pthread_attr_setdetachstate(&pAttrs, PTHREAD_CREATE_DETACHED);

    if (retc != 0)
    {
        gInitState &= ~MQTT_INIT_STATE;
        UART_PRINT("MQTT thread create fail\n\r");
        return;
    }

    retc = pthread_create(&mqttThread, &pAttrs, MqttClient, (void *) &threadArg);
    if (retc != 0)
    {
        gInitState &= ~MQTT_INIT_STATE;
        UART_PRINT("MQTT thread create fail\n\r");
        return;
    }

    /* enable interrupt for the GPIO 13 (SW3) and GPIO 22 (SW2).              */
    GPIO_setCallback(Board_BUTTON0, pushButtonInterruptHandler2);
    GPIO_enableInt(Board_BUTTON0); // SW2

    GPIO_setCallback(Board_BUTTON1, pushButtonInterruptHandler3);
    GPIO_enableInt(Board_BUTTON1); // SW3

    gInitState &= ~MQTT_INIT_STATE;

}

//*****************************************************************************
//!
//! MQTT Stop - Close the client instance and free all the items required to 
//! run the MQTT protocol
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************

void Mqtt_Stop()
{
    struct msgQueue queueElement;
    struct msgQueue queueElemRecv;

    if (gApConnectionState >= 0)
    {
        Mqtt_ClientStop(1);
    }

    queueElement.event = THREAD_TERMINATE_REQ;
    queueElement.msgPtr = NULL;

    /* write message indicating publish message                               */
    if (MQTT_SendMsgToQueue(&queueElement))
    {
        UART_PRINT("\n\n\rQueue is full, throw first msg and send the new one\n\n\r");
        mq_receive(g_PBQueue, (char*) &queueElemRecv, sizeof(struct msgQueue), NULL);
        MQTT_SendMsgToQueue(&queueElement);
    }

    sleep(2);

    mq_close(g_PBQueue);
    g_PBQueue = NULL;

    sl_Stop(200);
    UART_PRINT("\n\r Client Stop completed\r\n");

    /* Disable the SW2 and SW3 interrupt */
    GPIO_disableInt(Board_BUTTON0); // SW2
    GPIO_disableInt(Board_BUTTON1); // SW3
}

int32_t MqttClient_start()
{
    int32_t lRetVal = -1;
    int32_t iCount = 0;

    int32_t threadArg = 100;
    pthread_attr_t pAttrs;
    struct sched_param priParam;

    MqttClientExmple_params.clientId = ClientId;
    MqttClientExmple_params.connParams = &Mqtt_ClientCtx;
    MqttClientExmple_params.mqttMode31 = MQTT_3_1;
    MqttClientExmple_params.blockingSend = true;

    gInitState |= CLIENT_INIT_STATE;

    /* Initialize MQTT client lib                                             */
    gMqttClient = MQTTClient_create(MqttClientCallback, &MqttClientExmple_params);
    if( gMqttClient == NULL )
    {
        /* lib initialization failed                                          */
        gInitState &= ~CLIENT_INIT_STATE;
        return -1;
    }

    /* Open Client Receive Thread start the receive task. Set priority and    */
    /* stack size attributes                                                  */
    pthread_attr_init(&pAttrs);
    priParam.sched_priority = 2;
    lRetVal = pthread_attr_setschedparam(&pAttrs, &priParam);
    lRetVal |= pthread_attr_setstacksize(&pAttrs, RXTASKSIZE);
    lRetVal |= pthread_attr_setdetachstate(&pAttrs, PTHREAD_CREATE_DETACHED);
    lRetVal |= pthread_create(&g_rx_task_hndl, &pAttrs, MqttClientThread, (void *) &threadArg);
    if (lRetVal != 0)
    {
        UART_PRINT("Client Thread Create Failed failed\n\r");
        gInitState &= ~CLIENT_INIT_STATE;
        return -1;
    }
#ifdef SECURE_CLIENT
    setTime();
#endif

    /* setting will parameters                                                */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_WILL_PARAM, &will_param, sizeof(will_param));

#ifdef CLNT_USR_PWD
    /* Set user name for client connection                                    */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_USER_NAME, (void *)ClientUsername, strlen((char*)ClientUsername));

    /* Set password                                                           */
    MQTTClient_set(gMqttClient, MQTT_CLIENT_PASSWORD, (void *)ClientPassword, strlen((char*)ClientPassword));
#endif
    /* Initiate MQTT Connect                                                  */
    if (gApConnectionState >= 0)
    {
#if CLEAN_SESSION == false
        bool clean = CLEAN_SESSION;
        MQTTClient_set(gMqttClient, MQTT_CLIENT_CLEAN_CONNECT, (void *)&clean, sizeof(bool));
#endif
        /* The return code of MQTTClient_connect is the ConnACK value that
           returns from the server */
        lRetVal = MQTTClient_connect(gMqttClient);

        /* negative lRetVal means error,
           0 means connection successful without session stored by the server,
           greater than 0 means successful connection with session stored by
           the server */
        if (0 > lRetVal)
        {
            /* lib initialization failed                                      */
            UART_PRINT("Connection to broker failed\n\r");

            gUiConnFlag = 0;
        }
        else
        {
            gUiConnFlag = 1;
        }
        /* Subscribe to topics when session is not stored by the server       */
        if ( (gUiConnFlag == 1) && (0 == lRetVal) )
        {
            uint8_t subIndex;
            MQTTClient_SubscribeParams_t subscriptionInfo[SUBSCRIPTION_TOPIC_COUNT];

            for( subIndex = 0; subIndex < SUBSCRIPTION_TOPIC_COUNT; subIndex++ )
            {
                subscriptionInfo[subIndex].topic = topic[subIndex];
                subscriptionInfo[subIndex].qos = qos[subIndex];
            }

            if (MQTTClient_subscribe(gMqttClient , subscriptionInfo, SUBSCRIPTION_TOPIC_COUNT) < 0)
            {
                UART_PRINT("\n\r Subscription Error \n\r");
                MQTTClient_disconnect(gMqttClient);
                gUiConnFlag = 0;
            }
            else
            {
                for (iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++)
                {
                    UART_PRINT("Client subscribed on %s\n\r,", topic[iCount]);
                }
            }
        }
    }

    gInitState &= ~CLIENT_INIT_STATE;

    return 0;
}

//*****************************************************************************
//!
//! MQTT Client stop - Unsubscribe from the subscription topics and exit the 
//! MQTT client lib.
//!
//! \param  none
//!
//! \return None
//!
//*****************************************************************************

void Mqtt_ClientStop(uint8_t disconnect)
{
    uint32_t iCount;

    MQTTClient_UnsubscribeParams_t subscriptionInfo[SUBSCRIPTION_TOPIC_COUNT];

    for( iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++ )
    {
        subscriptionInfo[iCount].topic = topic[iCount];
    }

    MQTTClient_unsubscribe(gMqttClient , subscriptionInfo, SUBSCRIPTION_TOPIC_COUNT);
    for (iCount = 0; iCount < SUBSCRIPTION_TOPIC_COUNT; iCount++)
    {
        UART_PRINT("Unsubscribed from the topic %s\r\n", topic[iCount]);
    }
    gUiConnFlag = 0;

    /* exiting the Client library                                             */
    MQTTClient_delete(gMqttClient);

}

//*****************************************************************************
//!
//! Utility function which prints the borders
//!
//! \param[in] ch  -  hold the charater for the border.
//! \param[in] n   -  hold the size of the border.
//!
//! \return none.
//!
//*****************************************************************************

void printBorder(char ch, int n)
{
    int        i = 0;

    for(i=0; i<n; i++)    putch(ch);
}

//*****************************************************************************
//!
//! Set the ClientId with its own mac address
//! This routine converts the mac address which is given
//! by an integer type variable in hexadecimal base to ASCII
//! representation, and copies it into the ClientId parameter.
//!
//! \param  macAddress  -   Points to string Hex.
//!
//! \return void.
//!
//*****************************************************************************
void SetClientIdNamefromMacAddress(uint8_t *macAddress)
{
    uint8_t Client_Mac_Name[2];
    uint8_t Index;

    /* When ClientID isn't set, use the mac address as ClientID               */
    if (ClientId[0] == '\0')
    {
        /* 6 bytes is the length of the mac address                           */
        for (Index = 0; Index < SL_MAC_ADDR_LEN; Index++)
        {
            /* Each mac address byte contains two hexadecimal characters      */
            /* Copy the 4 MSB - the most significant character                */
            Client_Mac_Name[0] = (macAddress[Index] >> 4) & 0xf;
            /* Copy the 4 LSB - the least significant character               */
            Client_Mac_Name[1] = macAddress[Index] & 0xf;

            if (Client_Mac_Name[0] > 9)
            {
                /* Converts and copies from number that is greater than 9 in  */
                /* hexadecimal representation (a to f) into ascii character   */
                ClientId[2 * Index] = Client_Mac_Name[0] + 'a' - 10;
            }
            else
            {
                /* Converts and copies from number 0 - 9 in hexadecimal       */
                /* representation into ascii character                        */
                ClientId[2 * Index] = Client_Mac_Name[0] + '0';
            }
            if (Client_Mac_Name[1] > 9)
            {
                /* Converts and copies from number that is greater than 9 in  */
                /* hexadecimal representation (a to f) into ascii character   */
                ClientId[2 * Index + 1] = Client_Mac_Name[1] + 'a' - 10;
            }
            else
            {
                /* Converts and copies from number 0 - 9 in hexadecimal       */
                /* representation into ascii character                        */
                ClientId[2 * Index + 1] = Client_Mac_Name[1] + '0';
            }
        }
    }
}

//*****************************************************************************
//!
//! Utility function which Display the app banner
//!
//! \param[in] appName     -  holds the application name.
//! \param[in] appVersion  -  holds the application version.
//!
//! \return none.
//!
//*****************************************************************************

int32_t DisplayAppBanner(char* appName, char* appVersion)
{
    int32_t            ret = 0;
    uint8_t            macAddress[SL_MAC_ADDR_LEN];
    uint16_t           macAddressLen = SL_MAC_ADDR_LEN;
    uint16_t           ConfigSize = 0;
    uint8_t            ConfigOpt = SL_DEVICE_GENERAL_VERSION;
    SlDeviceVersion_t  ver = {0};

    ConfigSize = sizeof(SlDeviceVersion_t);

    /* Print device version info. */
    ret = sl_DeviceGet(SL_DEVICE_GENERAL, &ConfigOpt, &ConfigSize, (uint8_t*)(&ver));

    /* Print device Mac address */
    ret = sl_NetCfgGet(SL_NETCFG_MAC_ADDRESS_GET, 0, &macAddressLen, &macAddress[0]);

    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT("\t   %s Example Ver: %s",appName, appVersion);
    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);
    UART_PRINT("\t CHIP: 0x%x",ver.ChipId);
    UART_PRINT(lineBreak);
    UART_PRINT("\t MAC:  %d.%d.%d.%d",ver.FwVersion[0],ver.FwVersion[1],ver.FwVersion[2],ver.FwVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t PHY:  %d.%d.%d.%d",ver.PhyVersion[0],ver.PhyVersion[1],ver.PhyVersion[2],ver.PhyVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t NWP:  %d.%d.%d.%d",ver.NwpVersion[0],ver.NwpVersion[1],ver.NwpVersion[2],ver.NwpVersion[3]);
    UART_PRINT(lineBreak);
    UART_PRINT("\t ROM:  %d",ver.RomVersion);
    UART_PRINT(lineBreak);
    UART_PRINT("\t HOST: %s", SL_DRIVER_VERSION);
    UART_PRINT(lineBreak);
    UART_PRINT("\t MAC address: %02x:%02x:%02x:%02x:%02x:%02x", macAddress[0], macAddress[1], macAddress[2], macAddress[3], macAddress[4], macAddress[5]);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);
    UART_PRINT("\t");
    printBorder('=', 44);
    UART_PRINT(lineBreak);
    UART_PRINT(lineBreak);

    SetClientIdNamefromMacAddress(macAddress);

    return ret;
}

void mainThread(void * args)
{

    uint32_t count = 0;
    pthread_t spawn_thread = (pthread_t) NULL;
    pthread_attr_t pAttrs_spawn;
    struct sched_param priParam;
    int32_t retc = 0;
    UART_Handle tUartHndl;

    /* Call driver init functions */
    ADC_init();
    Board_initGPIO();
    Board_initSPI();

    /* Configure the UART                                                     */
    tUartHndl = InitTerm();
    /* remove uart receive from LPDS dependency                               */
    UART_control(tUartHndl, UART_CMD_RXDISABLE, NULL);

    char * timestamp = time_stamp();
    UART_PRINT("Current time: %s\n\n\r", timestamp);
    free(timestamp);


    /*  initialize PLL and relevant things here, found in the main thread of PLL.c  */

    UART_PRINT("Initializing PLL...\n\n\r");

    PLL_SPI = myInitSPI();
    if (PLL_SPI == NULL) {
        UART_PRINT("SPI handle was NULL, exiting... :(");
        return;
    }

    unsigned short checksums[9] = {0};
    float dispfreq;
    // uint32_t adc_sample = 0;
    init_PLL(PLL_SPI, checksums);
    /*  print out the resulting checksums and verify accuracy   */
    char checksum_echo[50] = "Checksums:";
    UART_PRINT("Checksum echo: %s\n\n\r", checksum_echo);
    sprintf(checksum_echo, "%d %d %d %d %d %d %d %d Total: %d\r\n",
            checksums[0], checksums[1], checksums[2], checksums[3],
            checksums[4], checksums[5], checksums[6], checksums[7],
            checksums[8]);
    UART_PRINT("Checksum echo: %s\n\n\r", checksum_echo);

    /*  tune to a starting frequency  (750 - 110 = 640 MHz)  */
    dispfreq = tune_PLL(750, PLL_SPI);

    /*  print   */
    UART_PRINT("PLL Tuned to %f\n\r", dispfreq);

    /* Create the sl_Task                                                     */
    pthread_attr_init(&pAttrs_spawn);
    priParam.sched_priority = SPAWN_TASK_PRIORITY;
    retc = pthread_attr_setschedparam(&pAttrs_spawn, &priParam);
    retc |= pthread_attr_setstacksize(&pAttrs_spawn, TASKSTACKSIZE);
    retc |= pthread_attr_setdetachstate(&pAttrs_spawn, PTHREAD_CREATE_DETACHED);

    retc = pthread_create(&spawn_thread, &pAttrs_spawn, sl_Task, NULL);

    if (retc != 0)
    {
        UART_PRINT("could not create simplelink task\n\r");
        while (1);
    }

    retc = sl_Start(0, 0, 0);
    if (retc < 0)
    {
        /* Handle Error */
        UART_PRINT("\n sl_Start failed\n");
        while(1);
    }

    /* Output device information to the UART terminal */
    retc = DisplayAppBanner(APPLICATION_NAME, APPLICATION_VERSION);

    retc = sl_Stop(SL_STOP_TIMEOUT );
    if (retc < 0)
    {
        /* Handle Error */
        UART_PRINT("\n sl_Stop failed\n");
        while(1);
    }

    if(retc < 0)
    {
        /* Handle Error */
        UART_PRINT("mqtt_client - Unable to retrieve device information \n");
        while (1);
    }


    while (1)
    {

        gResetApplication = false;
        topic[0] = SUBSCRIPTION_TOPIC0;
        topic[1] = SUBSCRIPTION_TOPIC1;
        topic[2] = SUBSCRIPTION_TOPIC2;
        topic[3] = SUBSCRIPTION_TOPIC3;
        gInitState = 0;

        /* Connect to AP                                                      */
        gApConnectionState = Mqtt_IF_Connect();

        gInitState |= MQTT_INIT_STATE;
        /* Run MQTT Main Thread (it will open the Client and Server)          */
        Mqtt_start();               

        /* Wait for init to be completed!!!                                   */
        while (gInitState != 0)
        {
            UART_PRINT(".");
            sleep(1);
        }
        UART_PRINT(".\r\n");

        while (gResetApplication == false) {
            if (scan_flag) {
                struct scan_vals scan_job = initScan(globalQueueElem, globalTopicOffset, globalPayloadOffset);
                if (scan_job.fmax && scan_job.fmin && scan_job.n_samples) {
                    UART_PRINT("About to scan..\n\n\r");
                    ScanHelper(scan_job.fmin, scan_job.fmax, scan_job.n_samples);
                }
                free(globalQueueElem->msgPtr);
                scan_flag = 0;
            }
        }

        UART_PRINT("TO Complete - Closing all threads and resources\r\n");

        /* Stop the MQTT Process                                              */
        Mqtt_Stop();

        UART_PRINT("reopen MQTT # %d  \r\n", ++count);

    }
}

//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
